{**********************************************************************}
{                                                                      }
{       Developer Express Visual Component Library                     }
{       ExpressSideBar - CLX/VCL Edition                               }
{                                                                      }
{       Copyright (c) 1998-2001 Developer Express Inc.                 }
{       ALL RIGHTS RESERVED                                            }
{                                                                      }
{   The entire contents of this file is protected by U.S. and          }
{   International Copyright Laws. Unauthorized reproduction,           }
{   reverse-engineering, and distribution of all or any portion of     }
{   the code contained in this file is strictly prohibited and may     }
{   result in severe civil and criminal penalties and will be          }
{   prosecuted to the maximum extent possible under the law.           }
{                                                                      }
{   RESTRICTIONS                                                       }
{                                                                      }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES              }
{   (DCU, OBJ, DLL, DPU, SO, ETC.) ARE CONFIDENTIAL AND PROPRIETARY    }
{   TRADE SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER   }
{   IS LICENSED TO DISTRIBUTE THE EXPRESSSIDEBAR                       }
{   AS PART OF AN EXECUTABLE PROGRAM ONLY.                             }
{                                                                      }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED         }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE           }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE          }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT     }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                         }
{                                                                      }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON          }
{   ADDITIONAL RESTRICTIONS.                                           }
{                                                                      }
{**********************************************************************}

unit cxControls;

{$I cxVer.inc}

interface

uses
  Classes,
{$IFDEF VCL}
  Windows, Messages, Controls;
{$ELSE}
  Qt, Types, QControls;
{$ENDIF}

{$IFNDEF VCL}
const
  VK_ESCAPE = Key_Escape;
  VK_RETURN = Key_Return;
  VK_INSERT = Key_Insert;
  VK_DELETE = Key_Delete;
{$ENDIF}

type
{$IFDEF VCL}
  TcxDragDetect = (ddNone, ddDrag, ddCancel);
{$ENDIF}

  TDragControlObjectClass = class of TDragControlObject;

  TcxControl = class(TCustomControl)
  private
    FMouseDownPos: TPoint;
    function GetIsCopyDragDrop: Boolean;
    function GetIsDesigning: Boolean;
    function GetIsDestroying: Boolean;
    function GetIsLoading: Boolean;
  {$IFDEF VCL}
    procedure WMEraseBkgnd(var Message: TWMEraseBkgnd); message WM_ERASEBKGND;
    procedure CMDesignHitTest(var Message: TCMDesignHitTest); message CM_DESIGNHITTEST;
    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
  {$ENDIF}
  protected
  {$IFDEF VCL}
    procedure CreateWnd; override;
    procedure WndProc(var Message: TMessage); override;
  {$ELSE}
    procedure BeginAutoDrag; override;
    function DesignEventQuery(Sender: QObjectH; Event: QEventH): Boolean; override;
    procedure InitWidget; override;
  {$ENDIF}
    procedure DoStartDrag(var DragObject: TDragObject); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

  {$IFDEF VCL}
    procedure FontChanged; dynamic;
    procedure MouseEnter(AControl: TControl); dynamic;
    procedure MouseLeave(AControl: TControl); dynamic;
  {$ENDIF}
    function AcceptMousePosForClick(X, Y: Integer): Boolean; virtual;
    function CanDrag(X, Y: Integer): Boolean; dynamic;
    function GetDesignHitTest(X, Y: Integer; Shift: TShiftState): Boolean; dynamic;
    function GetDragObjectClass: TDragControlObjectClass; dynamic;
    function HasBackground: Boolean; virtual;
    procedure InitControl; virtual;
    function MayFocus: Boolean; dynamic;
    function StartDrag(var DragObject: TDragObject): Boolean; dynamic;

    property IsCopyDragDrop: Boolean read GetIsCopyDragDrop;
    property IsDesigning: Boolean read GetIsDesigning;
    property IsDestroying: Boolean read GetIsDestroying;
    property IsLoading: Boolean read GetIsLoading;
    property MouseDownPos: TPoint read FMouseDownPos write FMouseDownPos;
  public
  {$IFDEF VCL}
    procedure InvalidateRect(const R: TRect; EraseBackground: Boolean);
  {$ENDIF}
  end;

  TcxDragControlObject = class(TDragControlObject)
  protected
  {$IFDEF VCL}
    function GetDragCursor(Accepted: Boolean; X, Y: Integer): TCursor; override;
  {$ENDIF}  
  end;

{$IFDEF VCL}
function DragDetect(Wnd: HWND): TcxDragDetect;
{$ENDIF}
function IsCtrlPressed: Boolean;
function TranslateKey(Key: Word): Word;

implementation

{$R cxControls.res}

uses
{$IFDEF VCL}
  Forms;
{$ELSE}
  QForms;
{$ENDIF}

{$IFDEF VCL}
const
  crDragCopy = 2000;
{$ENDIF}

{$IFDEF VCL}
function DragDetect(Wnd: HWND): TcxDragDetect;
var
  NoDragZone: TRect;
  P: TPoint;
  Msg: TMsg;
begin
  Result := ddCancel;

  GetCursorPos(P);
  with NoDragZone, P do
  begin
    Right := 2 * Mouse.DragThreshold;//GetSystemMetrics(SM_CXDRAG);
    Bottom := 2 * Mouse.DragThreshold;//GetSystemMetrics(SM_CYDRAG);
    Left := X - Right div 2;
    Top := Y - Bottom div 2;
    Inc(Right, Left);
    Inc(Bottom, Top);
  end;

  SetCapture(Wnd);
  try
    while GetCapture = Wnd do
    begin
      case Integer(GetMessage(Msg, 0, 0, 0)) of
        -1: Break;
        0: begin
            PostQuitMessage(Msg.wParam);
            Break;
          end;
      end;
      try
        case Msg.message of
          WM_KEYDOWN, WM_KEYUP:
            if Msg.wParam = VK_ESCAPE then Break;
          WM_MOUSEMOVE:
            begin
              P := SmallPointToPoint(TSmallPoint(Msg.lParam));
              ClientToScreen(Msg.hwnd, P);
              if not PtInRect(NoDragZone, P) then
              begin
                Result := ddDrag;
                Break;
              end;
            end;
          WM_LBUTTONUP:
            begin
              Result := ddNone;
              Break;
            end;
        end;
      finally
        TranslateMessage(Msg);
        DispatchMessage(Msg);
      end;
    end;
  finally
    if GetCapture = Wnd then ReleaseCapture;
  end;
end;
{$ENDIF}

function IsCtrlPressed: Boolean;
begin
{$IFDEF VCL}
  Result := GetAsyncKeyState(VK_CONTROL) < 0;
{$ELSE}
  Result := False;
{$ENDIF}
end;

function TranslateKey(Key: Word): Word;
begin
{$IFDEF VCL}
  Result := Key;
{$ELSE}
  if (Key = Key_Return) or (Key = Key_Enter) then
    Result := VK_RETURN
  else
    Result := Key;
{$ENDIF}
end;

{ TcxControl }

function TcxControl.GetIsCopyDragDrop: Boolean;
begin
  Result := IsCtrlPressed;
end;

function TcxControl.GetIsDesigning: Boolean;
begin
  Result := csDesigning in ComponentState;
end;

function TcxControl.GetIsDestroying: Boolean;
begin
  Result := csDestroying in ComponentState;
end;

function TcxControl.GetIsLoading: Boolean;
begin
  Result := csLoading in ComponentState;
end;

{$IFDEF VCL}
procedure TcxControl.WMEraseBkgnd(var Message: TWMEraseBkgnd);
begin
  if HasBackground then
    inherited
  else
    Message.Result := 1;
end;

procedure TcxControl.CMDesignHitTest(var Message: TCMDesignHitTest);
begin
  application.MessageBox('CMDesignHitTest','',0);
  with Message do
    Result := Integer(GetDesignHitTest(XPos, YPos, KeysToShiftState(Keys)));
end;

procedure TcxControl.CMMouseEnter(var Message: TMessage);
begin
  inherited;
  if Message.lParam = 0 then
    MouseEnter(Self)
  else
    MouseEnter(TControl(Message.lParam));
end;

procedure TcxControl.CMMouseLeave(var Message: TMessage);
begin
  inherited;
  if Message.lParam = 0 then
    MouseLeave(Self)
  else
    MouseLeave(TControl(Message.lParam));
end;
{$ENDIF}

{$IFDEF VCL}
procedure TcxControl.CreateWnd;
begin
  inherited;
  InitControl;
end;

procedure TcxControl.WndProc(var Message: TMessage);
begin
  with TWMMouse(Message) do
    if (DragMode = dmAutomatic) and not IsDesigning and not Dragging and
      ((Msg = WM_LBUTTONDOWN) or (Msg = WM_LBUTTONDBLCLK)) then
    begin
      if not IsControlMouseMsg(TWMMouse(Message)) then
      begin
        ControlState := ControlState + [csLButtonDown];
        Dispatch(Message);
      end;
      Exit;
    end;
  inherited;
end;

{$ELSE}

procedure TcxControl.BeginAutoDrag;
begin
  BeginDrag(False, Mouse.DragThreshold);
end;

function TcxControl.DesignEventQuery(Sender: QObjectH; Event: QEventH): Boolean;
var
  P: TPoint;
  //Control: TControl;
  ShiftState: TShiftState;
begin
  if QEvent_type(Event) in [QEventType_MouseButtonPress..QEventType_MouseMove] then
  begin
    P.X := QMouseEvent_x(QMouseEventH(Event));
    P.Y := QMouseEvent_y(QMouseEventH(Event));
    QWidget_mapToGlobal(QWidgetH(Sender), @P, @P);
    QWidget_mapFromGlobal(ChildHandle, @P, @P);
    {if (GetMouseGrabControl <> nil) and (GetMouseGrabControl.Parent = Self) then
      Control := GetMouseGrabControl
    else if GetMouseGrabWidget <> Self then
      Control := ControlAtPos(P, False)
    else
      Control := Self;
    if Control = nil then
      Control := Self
    else if Control <> Self then
      P := TranslatePoint(Self, Control, P);}
    ShiftState := ButtonStateToShiftState(QMouseEvent_state(QMouseEventH(Event)));

    Result := GetDesignHitTest(P.X, P.Y, ShiftState);
  end
  else
    Result := inherited DesignEventQuery(Sender, Event);
end;

procedure TcxControl.InitWidget;
begin
  inherited;
  if not CanFocus then
    QWidget_setFocusPolicy(Handle, QWidgetFocusPolicy_NoFocus);
  if not HasBackground then
    QWidget_setBackgroundMode(Handle, QWidgetBackgroundMode_NoBackground);
  InitControl;
end;
{$ENDIF}

procedure TcxControl.DoStartDrag(var DragObject: TDragObject);
{$IFNDEF VCL}
var
  ADragObject: TDragObject;
{$ENDIF}
begin
  inherited;
  DragObject := GetDragObjectClass.Create(Self);
  if not StartDrag(DragObject) then
  begin
  {$IFDEF VCL}
    DragObject.Free;
    DragObject := nil;
    CancelDrag;
  {$ELSE}
    ADragObject := DragObject;
    EndDrag(False);
    ADragObject.Free;
  {$ENDIF}
  end;
end;

procedure TcxControl.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  MouseDownPos := Point(X, Y);
  inherited;
{$IFDEF VCL}
  if (DragMode = dmAutomatic) and CanDrag(X, Y) and (DragDetect(Handle) = ddDrag) then
    BeginDrag(False);
{$ENDIF}
end;

{$IFDEF VCL}
procedure TcxControl.FontChanged;
begin
  Invalidate;
end;

procedure TcxControl.MouseEnter(AControl: TControl);
begin
end;

procedure TcxControl.MouseLeave(AControl: TControl);
begin
end;
{$ENDIF}

function TcxControl.AcceptMousePosForClick(X, Y: Integer): Boolean;
begin
  Result :=
    (DragMode = dmManual) or
    (Abs(X - MouseDownPos.X) < Mouse.DragThreshold) and
    (Abs(Y - MouseDownPos.Y) < Mouse.DragThreshold);
end;

function TcxControl.CanDrag(X, Y: Integer): Boolean;
begin
  Result := False;
end;

function TcxControl.GetDesignHitTest(X, Y: Integer; Shift: TShiftState): Boolean;
begin
  Result := False;
end;

function TcxControl.GetDragObjectClass: TDragControlObjectClass;
begin
  Result := TcxDragControlObject;
end;

function TcxControl.HasBackground: Boolean;
begin
  Result := False;
end;

procedure TcxControl.InitControl;
begin
end;

function TcxControl.MayFocus: Boolean;
begin
  Result := True;
end;

function TcxControl.StartDrag(var DragObject: TDragObject): Boolean;
begin
{$IFDEF VCL}
  Result := True;
{$ELSE}
  with MouseDownPos do
    Result := CanDrag(X, Y);
  // because of the bug in Kylix (MouseDown is not called next time after Drag&Drop)  
  if DragMode = dmAutomatic then Result := False;
{$ENDIF}
end;

{$IFDEF VCL}
procedure TcxControl.InvalidateRect(const R: TRect; EraseBackground: Boolean);
begin
  if HandleAllocated then
    Windows.InvalidateRect(Handle, @R, EraseBackground)
end;
{$ENDIF}

{ TcxDragControlObject }

{$IFDEF VCL}
function TcxDragControlObject.GetDragCursor(Accepted: Boolean; X, Y: Integer): TCursor;
begin
  if Accepted and (Control as TcxControl).IsCopyDragDrop then
    Result := crDragCopy
  else
    Result := inherited GetDragCursor(Accepted, X, Y);
end;
{$ENDIF}

{$IFDEF VCL}
initialization
  Screen.Cursors[crDragCopy] := LoadCursor(HInstance, 'CX_DRAGCOPYCURSOR');
{$ENDIF}

end.


