{**********************************************************************}
{                                                                      }
{       Developer Express Visual Component Library                     }
{       ExpressSideBar - CLX/VCL Edition                               }
{                                                                      }
{       Copyright (c) 1998-2001 Developer Express Inc.                 }
{       ALL RIGHTS RESERVED                                            }
{                                                                      }
{   The entire contents of this file is protected by U.S. and          }
{   International Copyright Laws. Unauthorized reproduction,           }
{   reverse-engineering, and distribution of all or any portion of     }
{   the code contained in this file is strictly prohibited and may     }
{   result in severe civil and criminal penalties and will be          }
{   prosecuted to the maximum extent possible under the law.           }
{                                                                      }
{   RESTRICTIONS                                                       }
{                                                                      }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES              }
{   (DCU, OBJ, DLL, DPU, SO, ETC.) ARE CONFIDENTIAL AND PROPRIETARY    }
{   TRADE SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER   }
{   IS LICENSED TO DISTRIBUTE THE EXPRESSSIDEBAR                       }
{   AS PART OF AN EXECUTABLE PROGRAM ONLY.                             }
{                                                                      }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED         }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE           }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE          }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT     }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                         }
{                                                                      }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON          }
{   ADDITIONAL RESTRICTIONS.                                           }
{                                                                      }
{**********************************************************************}

unit cxGraphics;

{$I cxVer.inc}

interface

uses
{$IFDEF VCL}
  Windows, Graphics, ImgList, Controls;
{$ELSE}
  Types, QGraphics, QImgList;
{$ENDIF}

const
  cxAlignLeft = 1;     { $1 }
  cxAlignRight = 2;    { $2 }
  cxAlignHCenter = 4;  { $4 }
  cxAlignTop = 8;      { $8 }
  cxAlignBottom = 16;  { $10 }
  cxAlignVCenter = 32; { $20 }
  cxAlignCenter = 36;  { $24 }
  cxSingleLine = 64;   { $40 }
  cxDontClip = 128;    { $80 }
  cxExpandTabs = 256;  { $100 }
  cxShowPrefix = 512;  { $200 }
  cxWordBreak = 1024;  { $400 }
  cxDontPrint = 4096;  { $1000 }

type
  TcxCanvas = class
  private
    FCanvas: TCanvas;
    
    function GetBrush: TBrush;
    function GetFont: TFont;
    function GetPen: TPen;
    procedure SetBrush(Value: TBrush);
    procedure SetFont(Value: TFont);
    procedure SetPen(Value: TPen);
  protected
    property Canvas: TCanvas read FCanvas;
  public
    constructor Create(ACanvas: TCanvas);

    procedure Draw(X, Y: Integer; Graphic: TGraphic);
    procedure DrawEdge(R: TRect; Sunken, Outer: Boolean);
    procedure DrawImage(Images: TImageList; X, Y, Index: Integer;
      Enabled: Boolean = True);
    procedure DrawText(const Text: string; R: TRect; Flags: Integer;
      Enabled: Boolean = True);
    procedure ExcludeClipRect(const R: TRect);
    procedure FillRect(const R: TRect);
    procedure FrameRect(const R: TRect; Color: TColor = clNone);
    procedure IntersectClipRect(const R: TRect);
    procedure Polygon(const Points: array of TPoint);
    procedure TextExtent(const Text: string; var R: TRect; Flags: Integer); //overload;
    function TextHeight(const Text: string): Integer;

    property Brush: TBrush read GetBrush write SetBrush;
    property Font: TFont read GetFont write SetFont;
    property Pen: TPen read GetPen write SetPen;
  end;

{$IFNDEF VCL}
function GetRValue(Color: Integer): Byte;
function GetGValue(Color: Integer): Byte;
function GetBValue(Color: Integer): Byte;
function RGB(R, G, B: Byte): Integer;
{$ENDIF}

implementation

uses
{$IFDEF VCL}
  Classes;
{$ELSE}
  Qt;
{$ENDIF}

{$IFDEF VCL}
function cxFlagsToDTFlags(Flags: Integer): Integer;
begin
  Result := DT_NOPREFIX;
  if cxAlignLeft and Flags <> 0 then
    Result := Result or DT_LEFT;
  if cxAlignRight and Flags <> 0 then
    Result := Result or DT_RIGHT;
  if cxAlignHCenter and Flags <> 0 then
    Result := Result or DT_CENTER;
  if cxAlignTop and Flags <> 0 then
    Result := Result or DT_TOP;
  if cxAlignBottom and Flags <> 0 then
    Result := Result or DT_BOTTOM;
  if cxAlignVCenter and Flags <> 0 then
    Result := Result or DT_VCENTER;
  if cxSingleLine and Flags <> 0 then
    Result := Result or DT_SINGLELINE;
  if cxDontClip and Flags <> 0 then
    Result := Result or DT_NOCLIP;
  if cxExpandTabs and Flags <> 0 then
    Result := Result or DT_EXPANDTABS;
  if cxShowPrefix and Flags <> 0 then
    Result := Result and not DT_NOPREFIX;
  if cxWordBreak and Flags <> 0 then
    Result := Result or DT_WORDBREAK or DT_EDITCONTROL;
  if cxDontPrint and Flags <> 0 then
    Result := Result or DT_CALCRECT;
end;
{$ELSE}
function GetRValue(Color: Integer): Byte;
begin
  Result := Byte(Color);
end;

function GetGValue(Color: Integer): Byte;
begin
  Result := Byte(Color shr 8);
end;

function GetBValue(Color: Integer): Byte;
begin
  Result := Byte(Color shr 16);
end;

function RGB(R, G, B: Byte): Integer;
begin
  Result := (R or (G shl 8) or (B shl 16));
end;
{$ENDIF}

{ TcxCanvas }

constructor TcxCanvas.Create(ACanvas: TCanvas);
begin
  inherited Create;
  FCanvas := ACanvas;
end;

function TcxCanvas.GetBrush: TBrush;
begin
  Result := Canvas.Brush;
end;

function TcxCanvas.GetFont: TFont;
begin
  Result := Canvas.Font;
end;

function TcxCanvas.GetPen: TPen;
begin
  Result := Canvas.Pen;
end;

procedure TcxCanvas.SetBrush(Value: TBrush);
begin
  Canvas.Brush := Value;
end;

procedure TcxCanvas.SetFont(Value: TFont);
begin
  Canvas.Font := Value;
end;

procedure TcxCanvas.SetPen(Value: TPen);
begin
  Canvas.Pen := Value;
end;

procedure TcxCanvas.Draw(X, Y: Integer; Graphic: TGraphic);
begin
  Canvas.Draw(X, Y, Graphic);
end;

procedure TcxCanvas.DrawEdge(R: TRect; Sunken, Outer: Boolean);

  procedure DrawOneEdge(LeftTopColor, RightBottomColor: TColor);
  begin
    with R do
    begin
      Brush.Color := RightBottomColor;
      FillRect(Rect(Left, Bottom - 1, Right, Bottom));
      FillRect(Rect(Right - 1, Top, Right, Bottom - 1));
      Brush.Color := LeftTopColor;
      FillRect(Rect(Left, Top, Left + 1, Bottom - 1));
      FillRect(Rect(Left + 1, Top, Right - 1, Top + 1));
    end;
  end;

begin
  if Sunken then
    if Outer then
      DrawOneEdge(clBtnShadow, clBtnHighlight)
    else
      DrawOneEdge(clBtnText, clBtnFace)
  else
    if Outer then
      DrawOneEdge(clBtnFace, clBtnText)
    else
      DrawOneEdge(clBtnHighlight, clBtnShadow);
end;

procedure TcxCanvas.DrawImage(Images: TImageList; X, Y, Index: Integer;
  Enabled: Boolean = True);
begin
  if (0 <= Index) and (Index < Images.Count) then
    Images.Draw(Canvas, X, Y, Index, {$IFNDEF VCL}itImage, {$ENDIF}Enabled);
end;

procedure TcxCanvas.DrawText(const Text: string; R: TRect; Flags: Integer;
  Enabled: Boolean);
var
  PrevFontColor: TColor;
begin
{$IFDEF VCL}
  Flags := cxFlagsToDTFlags(Flags);
{$ENDIF}
  PrevFontColor := Font.Color;
  if not Enabled then
  begin
    with R do
    begin
      Inc(Left);
      Inc(Top);
    end;
    Font.Color := clBtnHighlight;
  {$IFDEF VCL}
    Windows.DrawText(Canvas.Handle, PChar(Text), Length(Text), R, Flags);
  {$ELSE}
    Canvas.TextRect(R, R.Left, R.Top, Text, Flags);
  {$ENDIF}
    OffsetRect(R, -1, -1);
    Font.Color := clBtnShadow;
  end;
{$IFDEF VCL}
  Windows.DrawText(Canvas.Handle, PChar(Text), Length(Text), R, Flags);
{$ELSE}
  Canvas.TextRect(R, R.Left, R.Top, Text, Flags);
{$ENDIF}
  Font.Color := PrevFontColor;
end;

procedure TcxCanvas.ExcludeClipRect(const R: TRect);
{$IFDEF VCL}
begin
  with R do
    Windows.ExcludeClipRect(Canvas.Handle, Left, Top, Right, Bottom);
end;
{$ELSE}
var
  NewR: TRect;
  ClipRgn, Rgn: QRegionH;
begin
  NewR := R;
{$IFDEF WINDOWS}
  with NewR do
  begin
    Dec(Right);
    Dec(Bottom);
  end;
{$ENDIF}
  ClipRgn := Canvas.GetClipRegion;
  Rgn := QRegion_create(@NewR, QRegionRegionType_Rectangle);
  QRegion_subtract(ClipRgn, ClipRgn, Rgn);
  QRegion_destroy(Rgn);
  QPainter_setClipRegion(Canvas.Handle, ClipRgn);
end;
{$ENDIF}

procedure TcxCanvas.FillRect(const R: TRect);
{$IFDEF VCL}
begin
  Canvas.FillRect(R);
end;
{$ELSE}
var
  NewR: TRect;
begin
  NewR := R;
{$IFDEF WINDOWS}
  with NewR do
  begin
    Dec(Right);
    Dec(Bottom);
  end;
{$ENDIF}
  Canvas.FillRect(NewR);
end;
{$ENDIF}

procedure TcxCanvas.FrameRect(const R: TRect; Color: TColor = clNone);
begin
  if Color <> clNone then
  begin
    Brush.Color := Color;
    Brush.Style := bsSolid;  // bug in Qt
  end;
  with R do
  begin
    FillRect(Rect(Left, Top, Left + 1, Bottom));
    FillRect(Rect(Right - 1, Top, Right, Bottom));
    FillRect(Rect(Left + 1, Top, Right - 1, Top + 1));
    FillRect(Rect(Left + 1, Bottom - 1, Right - 1, Bottom));
  end;
end;

procedure TcxCanvas.IntersectClipRect(const R: TRect);
{$IFDEF VCL}
begin
  with R do
    Windows.IntersectClipRect(Canvas.Handle, Left, Top, Right, Bottom);
end;
{$ELSE}
var
  NewR: TRect;
  ClipRgn, Rgn: QRegionH;
begin
  NewR := R;
{$IFDEF WINDOWS}
  with NewR do
  begin
    Dec(Right);
    Dec(Bottom);
  end;
{$ENDIF}
  ClipRgn := Canvas.GetClipRegion;
  Rgn := QRegion_create(@NewR, QRegionRegionType_Rectangle);
  QRegion_intersect(ClipRgn, ClipRgn, Rgn);
  QRegion_destroy(Rgn);
  QPainter_setClipRegion(Canvas.Handle, ClipRgn);
end;
{$ENDIF}

procedure TcxCanvas.Polygon(const Points: array of TPoint);
begin
  Canvas.Polygon(Points{$IFNDEF VCL}, False, 0, -1{$ENDIF});
end;

procedure TcxCanvas.TextExtent(const Text: string; var R: TRect; Flags: Integer);
{$IFDEF VCL}
var
  RWidth, RHeight, TextWidth, TextHeight: Integer;

  procedure CalcRSizes(var AWidth, AHeight: Integer);
  begin
    with R do
    begin
      AWidth := Right - Left;
      AHeight := Bottom - Top;
    end;
  end;

  procedure AlignR;
  begin
    if Flags and DT_CENTER <> 0 then
      OffsetRect(R, (RWidth - TextWidth) div 2, 0)
    else
      if Flags and DT_RIGHT <> 0 then
        OffsetRect(R, RWidth - TextWidth, 0);
    if Flags and DT_VCENTER <> 0 then
      OffsetRect(R, 0, (RHeight - TextHeight) div 2)
    else
      if Flags and DT_BOTTOM <> 0 then
        OffsetRect(R, 0, RHeight - TextHeight);
  end;

begin
  CalcRSizes(RWidth, RHeight);
  Flags := cxFlagsToDTFlags(Flags) or DT_CALCRECT;
  Windows.DrawText(Canvas.Handle, PChar(Text), Length(Text), R, Flags);
  CalcRSizes(TextWidth, TextHeight);
  AlignR;
end;
{$ELSE}
begin
  Canvas.TextExtent(Text, R, Flags);
{$IFDEF WINDOWS}
  with R do
  begin
    Inc(Right);
    Inc(Bottom);
  end;
{$ENDIF}
end;
{$ENDIF}

function TcxCanvas.TextHeight(const Text: string): Integer;
begin
  Result := Canvas.TextHeight(Text);
end;

end.
