{**********************************************************************}
{                                                                      }
{       Developer Express Visual Component Library                     }
{       ExpressSideBar - CLX/VCL Edition                               }
{                                                                      }
{       Copyright (c) 1998-2001 Developer Express Inc.                 }
{       ALL RIGHTS RESERVED                                            }
{                                                                      }
{   The entire contents of this file is protected by U.S. and          }
{   International Copyright Laws. Unauthorized reproduction,           }
{   reverse-engineering, and distribution of all or any portion of     }
{   the code contained in this file is strictly prohibited and may     }
{   result in severe civil and criminal penalties and will be          }
{   prosecuted to the maximum extent possible under the law.           }
{                                                                      }
{   RESTRICTIONS                                                       }
{                                                                      }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES              }
{   (DCU, OBJ, DLL, DPU, SO, ETC.) ARE CONFIDENTIAL AND PROPRIETARY    }
{   TRADE SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER   }
{   IS LICENSED TO DISTRIBUTE THE EXPRESSSIDEBAR                       }
{   AS PART OF AN EXECUTABLE PROGRAM ONLY.                             }
{                                                                      }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED         }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE           }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE          }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT     }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                         }
{                                                                      }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON          }
{   ADDITIONAL RESTRICTIONS.                                           }
{                                                                      }
{**********************************************************************}

unit cxSideBar;

{$I cxSideBarVer.inc}

interface

uses
  SysUtils, Classes,
{$IFDEF VCL}
  Windows, Messages, Graphics, Controls, Forms, ImgList, StdCtrls, ExtCtrls,
{$ELSE}
  Types, Qt, QGraphics, QControls, QForms, QImgList, QStdCtrls, QExtCtrls,
{$ENDIF}
  cxGraphics, cxControls;

type
  TcxSideBarItem = class;
  TcxSideBarLink = class;
  TcxSideBarLinks = class;
  TcxSideBarGroup = class;
  TcxSideBarInplaceEdit = class;
  TcxSideBarViewInfo = class;
  TcxSideBar = class;

  TDesigner = {$IFDEF DELPHI6}IDesignerHook{$ELSE}IDesigner{$ENDIF};

  TcxSideBarDesignerLink = class
  protected
    function GetDesigner(AControl: TcxSideBar): TDesigner;
  public
    procedure ControlNameChanged(AControl: TcxSideBar); virtual;
    procedure GroupNameChanged(AControl: TcxSideBar); virtual;
    procedure ItemNameChanged(AControl: TcxSideBar); virtual;
    procedure LinkItemChanged(AControl: TcxSideBar); virtual;
    procedure Modified(AControl: TcxSideBar);
    procedure SetSelection(AControl: TcxSideBar; AObject: TPersistent); virtual;
    procedure ShowDefaultEventHandler(AItem: TcxSideBarItem); virtual;
  end;

  TcxSideBarHitTestCode = (htNone, htBackground, htGroup,
    htLinkImage, htLinkCaption, htLinkInsertionPos, htScrollUp, htScrollDown);

  TcxSideBarViewState = (vsNone, vsSelected, vsPressed);

  TcxSideBarItem = class(TComponent)
  private
    FCaption: string;
    FControl: TcxSideBar;
    FEnabled: Boolean;
    FLargeImageIndex: Integer;
    FLinks: TList;
    FSmallImageIndex: Integer;
    FVisible: Boolean;
    FOnClick: TNotifyEvent;

    function GetActive: Boolean;
    function GetIndex: Integer;
    function GetLink(Index: Integer): TcxSideBarLink;
    function GetLinkCount: Integer;
    function GetVisible: Boolean;
    procedure SetActive(Value: Boolean);
    procedure SetCaption(const Value: string);
    procedure SetControl(Value: TcxSideBar);
    procedure SetEnabled(Value: Boolean);
    procedure SetIndex(Value: Integer);
    procedure SetLargeImageIndex(Value: Integer);
    procedure SetSmallImageIndex(Value: Integer);
    procedure SetVisible(Value: Boolean);
  protected
    procedure SetName(const NewName: TComponentName); override;
    procedure SetParentComponent(Value: TComponent); override;

    procedure AddLink(ALink: TcxSideBarLink);
    procedure DestroyLinks;
    procedure RemoveLink(ALink: TcxSideBarLink);

    procedure ActiveChanged;
    procedure Changed(ViewInfoChanged: Boolean);
    procedure DoClick; dynamic;
    procedure VisibleChanged;

    property LinkCount: Integer read GetLinkCount;
    property Links[Index: Integer]: TcxSideBarLink read GetLink;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetParentComponent: TComponent; override;
    function HasParent: Boolean; override;

    property Active: Boolean read GetActive write SetActive;
    property Control: TcxSideBar read FControl write SetControl;
    property Index: Integer read GetIndex write SetIndex;
  published
    property Caption: string read FCaption write SetCaption;
    property Enabled: Boolean read FEnabled write SetEnabled default True;
    property LargeImageIndex: Integer read FLargeImageIndex write SetLargeImageIndex default -1;
    property SmallImageIndex: Integer read FSmallImageIndex write SetSmallImageIndex default -1;
    property Visible: Boolean read GetVisible write SetVisible default True;

    property OnClick: TNotifyEvent read FOnClick write FOnClick;
  end;

  TcxSideBarLinkAssignedValue = (avCaption);
  TcxSideBarLinkAssignedValues = set of TcxSideBarLinkAssignedValue;

  TcxSideBarLink = class(TCollectionItem)
  private
    FAssignedValues: TcxSideBarLinkAssignedValues;
    FCaption: string;
    FItem: TcxSideBarItem;
    function GetActive: Boolean;
    function GetCaption: string;
    function GetControl: TcxSideBar;
    function GetEnabled: Boolean;
    function GetGroup: TcxSideBarGroup;
    function GetImageIndex: Integer;
    function GetImages: TImageList;
    function GetLinks: TcxSideBarLinks;
    function GetPressed: Boolean;
    function GetSelected: Boolean;
    function GetViewState: TcxSideBarViewState;
    function GetVisible: Boolean;
    function GetVisibleIndex: Integer;
    function GetVisuallyVisible: Boolean;
    procedure SetAssignedValues(Value: TcxSideBarLinkAssignedValues);
    procedure SetCaption(const Value: string);
    procedure SetItem(Value: TcxSideBarItem);
    procedure SetVisibleIndex(Value: Integer);
    function IsCaptionAssigned: Boolean;
  protected
    procedure CaptionChanged;
    procedure Changed(ViewInfoChanged: Boolean);
    procedure PressedChanged;
    procedure SelectedChanged;
    procedure ViewStateChanged;
    procedure VisibleChanged;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;

    procedure MakeVisuallyVisible;
    procedure StartEditing;

    property Active: Boolean read GetActive;
    property Control: TcxSideBar read GetControl;
    property Enabled: Boolean read GetEnabled;
    property Group: TcxSideBarGroup read GetGroup;
    property ImageIndex: Integer read GetImageIndex;
    property Images: TImageList read GetImages;
    property Links: TcxSideBarLinks read GetLinks;
    property Pressed: Boolean read GetPressed;
    property Selected: Boolean read GetSelected;
    property ViewState: TcxSideBarViewState read GetViewState;
    property Visible: Boolean read GetVisible;
    property VisibleIndex: Integer read GetVisibleIndex write SetVisibleIndex;
    property VisuallyVisible: Boolean read GetVisuallyVisible;
  published
    property AssignedValues: TcxSideBarLinkAssignedValues read FAssignedValues
      write SetAssignedValues stored False;
    property Caption: string read GetCaption write SetCaption stored IsCaptionAssigned;
    property Item: TcxSideBarItem read FItem write SetItem;
  end;

  TcxSideBarLinks = class(TCollection)
  private
    FGroup: TcxSideBarGroup;
    FVisibleItems: TList;
    function GetControl: TcxSideBar;
    function GetItem(Index: Integer): TcxSideBarLink;
    function GetVisibleCount: Integer;
    function GetVisibleItem(Index: Integer): TcxSideBarLink;
    procedure SetItem(Index: Integer; Value: TcxSideBarLink);
  protected
    function GetOwner: TPersistent; override;
    procedure Update(Item: TCollectionItem); override;

    procedure DoLinkDeleted(ALink: TcxSideBarLink);
    procedure RefreshVisibleItems;
    procedure VisibleItemsChanged;
  public
    constructor Create(AGroup: TcxSideBarGroup);
    destructor Destroy; override;
    function Add: TcxSideBarLink;

    property Control: TcxSideBar read GetControl;
    property Group: TcxSideBarGroup read FGroup;
    property Items[Index: Integer]: TcxSideBarLink read GetItem write SetItem; default;
    property VisibleCount: Integer read GetVisibleCount;
    property VisibleItems[Index: Integer]: TcxSideBarLink read GetVisibleItem;
  end;

  TcxSideBarStyle = (ssLargeImages, ssSmallImages);

  TcxSideBarGroupAssignedValue = (avStyle);
  TcxSideBarGroupAssignedValues = set of TcxSideBarGroupAssignedValue;

  TcxSideBarGroup = class(TComponent)
  private
    FAssignedValues: TcxSideBarGroupAssignedValues;
    FCaption: string;
    FControl: TcxSideBar;
    FLinks: TcxSideBarLinks;
    FStyle: TcxSideBarStyle;
    FVisible: Boolean;

    function GetActive: Boolean;
    function GetIndex: Integer;
    function GetPressed: Boolean;
    function GetSelected: Boolean;
    function GetStyle: TcxSideBarStyle;
    function GetViewState: TcxSideBarViewState;
    function GetVisible: Boolean;
    function GetVisibleIndex: Integer;
    function GetVisuallyVisible: Boolean;
    procedure SetActive(Value: Boolean);
    procedure SetAssignedValues(Value: TcxSideBarGroupAssignedValues);
    procedure SetCaption(const Value: string);
    procedure SetControl(Value: TcxSideBar);
    procedure SetIndex(Value: Integer);
    procedure SetLinks(Value: TcxSideBarLinks);
    procedure SetStyle(Value: TcxSideBarStyle);
    procedure SetVisible(Value: Boolean);

    function IsStyleAssigned: Boolean;
  protected
    procedure SetName(const NewName: TComponentName); override;
    procedure SetParentComponent(Value: TComponent); override;

    procedure Changed(ViewInfoChanged: Boolean);
    procedure PressedChanged;
    procedure SelectedChanged;
    procedure StyleChanged;
    procedure ViewStateChanged;
    procedure VisibleChanged;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function GetParentComponent: TComponent; override;
    function HasParent: Boolean; override;

    function AddLink(AItem: TcxSideBarItem): TcxSideBarLink;
    procedure StartEditing;

    property Active: Boolean read GetActive write SetActive;
    property Control: TcxSideBar read FControl write SetControl;
    property Index: Integer read GetIndex write SetIndex;
    property Pressed: Boolean read GetPressed;
    property Selected: Boolean read GetSelected;
    property ViewState: TcxSideBarViewState read GetViewState;
    property VisibleIndex: Integer read GetVisibleIndex;
    property VisuallyVisible: Boolean read GetVisuallyVisible;
  published
    property AssignedValues: TcxSideBarGroupAssignedValues read FAssignedValues
      write SetAssignedValues stored False;
    property Caption: string read FCaption write SetCaption;
    property Links: TcxSideBarLinks read FLinks write SetLinks;
    property Style: TcxSideBarStyle read GetStyle write SetStyle
      stored IsStyleAssigned;
    property Visible: Boolean read GetVisible write SetVisible default True;
  end;

  TcxSideBarScroll = class
  private
    FControl: TcxSideBar;
    FViewState: TcxSideBarViewState;
    procedure SetViewState(Value: TcxSideBarViewState);
  protected
    procedure Changed(ViewInfoChanged: Boolean);
    procedure DoClick; dynamic; abstract;
    function GetHitTestCode: TcxSideBarHitTestCode; virtual; abstract;
    function GetIsScrollUp: Boolean; virtual; abstract;
    function GetVisible: Boolean; virtual; abstract;
  public
    constructor Create(AControl: TcxSideBar); virtual;
    property Control: TcxSideBar read FControl;
    property IsScrollUp: Boolean read GetIsScrollUp;
    property ViewState: TcxSideBarViewState read FViewState write SetViewState;
    property Visible: Boolean read GetVisible;
  end;

  TcxSideBarScrollDown = class(TcxSideBarScroll)
  protected
    procedure DoClick; override;
    function GetHitTestCode: TcxSideBarHitTestCode; override;
    function GetIsScrollUp: Boolean; override;
    function GetVisible: Boolean; override;
  end;

  TcxSideBarScrollUp = class(TcxSideBarScroll)
  protected
    procedure DoClick; override;
    function GetHitTestCode: TcxSideBarHitTestCode; override;
    function GetIsScrollUp: Boolean; override;
    function GetVisible: Boolean; override;
  end;

  TcxSideBarBackgroundMode = (bmNone, bmTexture, bmGradientHorz, bmGradientVert);

  TcxSideBarBackground = class(TPersistent)
  private
    FBeginColor: TColor;
    FControl: TcxSideBar;
    FEndColor: TColor;
    FMiddleColor: TColor;
    FMode: TcxSideBarBackgroundMode;
    FTexture: TBitmap;
    procedure SetBeginColor(Value: TColor);
    procedure SetEndColor(Value: TColor);
    procedure SetMiddleColor(Value: TColor);
    procedure SetMode(Value: TcxSideBarBackgroundMode);
    procedure SetTexture(Value: TBitmap);
    procedure TextureChanged(Sender: TObject);
  protected
    procedure Changed;
  public
    constructor Create(AControl: TcxSideBar);
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    property Control: TcxSideBar read FControl;
  published
    property BeginColor: TColor read FBeginColor write SetBeginColor default clBlue;
    property EndColor: TColor read FEndColor write SetEndColor default clBlack;
    property MiddleColor: TColor read FMiddleColor write SetMiddleColor default clNone;
    property Mode: TcxSideBarBackgroundMode read FMode write SetMode default bmNone;
    property Texture: TBitmap read FTexture write SetTexture;
  end;

  TcxSideBarEditLink = class
  private
    FInplaceEdit: TcxSideBarInplaceEdit;
  protected
    property InplaceEdit: TcxSideBarInplaceEdit read FInplaceEdit;
  public
    constructor Create(AInplaceEdit: TcxSideBarInplaceEdit;
      AEditingObject: TObject); virtual;
    procedure Initiate; virtual; abstract;
    procedure StoreValue; virtual; abstract;
  end;

  TcxSideBarEditLinkForLink = class(TcxSideBarEditLink)
  private
    FLink: TcxSideBarLink;
  protected
    property Link: TcxSideBarLink read FLink;
  public
    constructor Create(AInplaceEdit: TcxSideBarInplaceEdit;
      AEditingObject: TObject); override;
    procedure Initiate; override;
    procedure StoreValue; override;
  end;

  TcxSideBarEditLinkForGroup = class(TcxSideBarEditLink)
  private
    FGroup: TcxSideBarGroup;
  protected
    property Group: TcxSideBarGroup read FGroup;
  public
    constructor Create(AInplaceEdit: TcxSideBarInplaceEdit;
      AEditingObject: TObject); override;
    procedure Initiate; override;
    procedure StoreValue; override;
  end;

  TcxSideBarInplaceEdit = class(TCustomMemo)
  private
    FControl: TcxSideBar;
    FEditLink: TcxSideBarEditLink;
    FExpandable: Boolean;
    FOriginalBounds: TRect;
    function GetBounds: TRect;
    procedure SetEditingObject(Value: TObject);
    procedure SetExpandable(Value: Boolean);
    procedure ChangeHandler(Sender: TObject);
  {$IFDEF VCL}
    procedure WMKillFocus(var Message: TMessage); message WM_KILLFOCUS;
  {$ENDIF}
  protected
    procedure Change; override;
  {$IFNDEF VCL}
    function EventFilter(Sender: QObjectH; Event: QEventH): Boolean; override;
  {$ENDIF}
    procedure KeyDown(var Key: Word; Shift: TShiftState); override;

    procedure DoFocusOut;

    property Bounds: TRect read GetBounds;
    property EditLink: TcxSideBarEditLink read FEditLink;
    property Expandable: Boolean read FExpandable write SetExpandable;
    property OriginalBounds: TRect read FOriginalBounds write FOriginalBounds;
  public
    constructor Create(AControl: TcxSideBar); reintroduce;
    procedure StartEditing;
    procedure FinishEditing(Accept: Boolean);

    property Control: TcxSideBar read FControl;
    property EditingObject: TObject write SetEditingObject;
  end;

  TcxSideBarGroupViewInfo = class
  private
    FBounds: TRect;
    FViewInfo: TcxSideBarViewInfo;
  public
    constructor Create(AViewInfo: TcxSideBarViewInfo);
    function WantsMouse(const P: TPoint): Boolean;
    property Bounds: TRect read FBounds write FBounds;
  end;
  PcxSideBarGroupsViewInfo = ^TcxSideBarGroupsViewInfo;
  TcxSideBarGroupsViewInfo =
    array[0..MaxInt div SizeOf(TcxSideBarGroupViewInfo) - 1] of TcxSideBarGroupViewInfo;

  TcxSideBarLinkViewInfo = class
  private
    FViewInfo: TcxSideBarViewInfo;
  public
    CaptionBounds: TRect;
    CaptionBoundsForMouse: TRect;
    ImageBounds: TRect;
    constructor Create(AViewInfo: TcxSideBarViewInfo);
    function WantsMouse(const P: TPoint; var OverImage: Boolean): Boolean;
  end;
  PcxSideBarLinksViewInfo = ^TcxSideBarLinksViewInfo;
  TcxSideBarLinksViewInfo =
    array[0..MaxInt div SizeOf(TcxSideBarLinkViewInfo) - 1] of TcxSideBarLinkViewInfo;

  TcxSideBarHitTestInfo = record
    Code: TcxSideBarHitTestCode;
    Group: TcxSideBarGroup;
    Link: TcxSideBarLink;
  end;

  TcxSideBarViewInfo = class
  private
    FActiveGroupContentBounds: TRect;
    FCanScrollDown: Boolean;
    FCanScrollUp: Boolean;
    FControl: TcxSideBar;
    FGroupsViewInfo: PcxSideBarGroupsViewInfo;
    FLinksViewInfo: PcxSideBarLinksViewInfo;
    FStoredGroupCount: Integer;
    FStoredLinkCount: Integer;
    FVisibleLinkCount: Integer;

    function GetBorderSize: Integer;
    function GetBounds: TRect;
    function GetCanvas: TcxCanvas;
    function GetFlat: Boolean;
    function GetGroupBorderSize: Integer;
    function GetGroupViewInfo(Index: Integer): TcxSideBarGroupViewInfo;
    function GetLinksAreaBounds: TRect;
    function GetLinkViewInfo(Index: Integer): TcxSideBarLinkViewInfo;
    function GetScrollDownBounds: TRect;
    function GetScrollUpBounds: TRect;
  protected
    procedure CalcGroupsViewInfo;
    procedure CalcLinksViewInfo;
    procedure CreateViewInfo;
    procedure DestroyViewInfo;
    function GetImageSize(AGroup: TcxSideBarGroup): TPoint;

    property Canvas: TcxCanvas read GetCanvas;
    property StoredGroupCount: Integer read FStoredGroupCount;
    property StoredLinkCount: Integer read FStoredLinkCount;
  public
    constructor Create(AControl: TcxSideBar);
    destructor Destroy; override;
    procedure CalcViewInfo;
    function GetHitTestInfo(X, Y: Integer): TcxSideBarHitTestInfo;
    function GetLinkInsertionPosBounds(Pos: Integer): TRect;
    function GetRect(Group: TcxSideBarGroup; Link: TcxSideBarLink;
      Part: TcxSideBarHitTestCode): TRect;
    function GetScrollBounds(AScroll: TcxSideBarSCroll): TRect;

    property ActiveGroupContentBounds: TRect read FActiveGroupContentBounds;
    property BorderSize: Integer read GetBorderSize;
    property Bounds: TRect read GetBounds;
    property CanScrollDown: Boolean read FCanScrollDown;
    property CanScrollUp: Boolean read FCanScrollUp;
    property Control: TcxSideBar read FControl;
    property Flat: Boolean read GetFlat;
    property GroupBorderSize: Integer read GetGroupBorderSize;
    property GroupViewInfos[Index: Integer]: TcxSideBarGroupViewInfo read
      GetGroupViewInfo;
    property LinksAreaBounds: TRect read GetLinksAreaBounds;
    property LinkViewInfos[Index: Integer]: TcxSideBarLinkViewInfo read
      GetLinkViewInfo;
    property ScrollDownBounds: TRect read GetScrollDownBounds;
    property ScrollUpBounds: TRect read GetScrollUpBounds;
    property VisibleLinkCount: Integer read FVisibleLinkCount;
  end;

  TcxSideBarPainterClass = class of TcxSideBarPainter;
  TcxSideBarPainter = class
  private
    FControl: TcxSideBar;
    function GetBackground: TcxSideBarBackground;
    function GetCanvas: TcxCanvas;
    function GetFlat: Boolean;
    function GetViewInfo: TcxSideBarViewInfo;
  protected
    procedure DrawBackground; virtual;
    procedure DrawBackgroundNone(const R: TRect); virtual;
    procedure DrawBackgroundTexture(const R: TRect); virtual;
    procedure DrawBorder; virtual;
    procedure DrawGradient(R: TRect; Vertical: Boolean; FromColor, ToColor: TColor);
    procedure DrawGradientHorz(const R: TRect); virtual;
    procedure DrawGradientVert(const R: TRect); virtual;
    procedure DrawGroup(AGroup: TcxSideBarGroup); virtual;
    procedure DrawGroups; virtual;
    procedure DrawItemBorder(var R: TRect; ViewState: TcxSideBarViewState;
      ReverseOrder: Boolean); virtual;
    procedure DrawLink(ALink: TcxSideBarLink); virtual;
    procedure DrawLinkCaption(ALink: TcxSideBarLink); virtual;
    procedure DrawLinkImage(ALink: TcxSideBarLink); virtual;
    procedure DrawLinkInsertionPos; virtual;
    procedure DrawLinks; virtual;
    procedure DrawScroll(AScroll: TcxSideBarScroll); virtual;
    procedure DrawScrollArrow(const R: TRect; IsScrollUp, IsPressed: Boolean);
    procedure DrawScrolls; virtual;
  public
    constructor Create(AControl: TcxSideBar); virtual;
    procedure Invalidate(Group: TcxSideBarGroup; Link: TcxSideBarLink;
      Part: TcxSideBarHitTestCode);
    procedure InvalidateLinkImageBorder(Link: TcxSideBarLink);
    procedure Paint; virtual;

    property Background: TcxSideBarBackground read GetBackground;
    property Canvas: TcxCanvas read GetCanvas;
    property Control: TcxSideBar read FControl;
    property Flat: Boolean read GetFlat;
    property ViewInfo: TcxSideBarViewInfo read GetViewInfo;
  end;

  TcxSideBarAssignedValue = (avGroupFont);
  TcxSideBarAssignedValues = set of TcxSideBarAssignedValue;
  TcxSideBarLookAndFeel = (sblfStandard, sblfFlat);
  TcxSideBarOption = (soAnimation, soHotTrack, soStayPressed);
  TcxSideBarOptions = set of TcxSideBarOption;

  TcxSideBar = class(TcxControl)
  private
    FActiveGroup: TcxSideBarGroup;
    FActiveItem: TcxSideBarItem;
    FAssignedValues: TcxSideBarAssignedValues;
    FAssignedValuesChanging: Boolean;
    FBackground: TcxSideBarBackground;
    FCanvas: TcxCanvas;
    FDraggingLink: TcxSideBarLink;
    FGroupFont: TFont;
    FGroups: TList;
    FInplaceEdit: TcxSideBarInplaceEdit;
    FItems: TList;
    FLargeImages: TImageList;
    FLargeImagesChangeLink: TChangeLink;
    FLastPressedLink: TcxSideBarLink;
    FLinkInsertionPos: Integer;
    FLookAndFeel: TcxSideBarLookAndFeel;
    FOptions: TcxSideBarOptions;
    FPainter: TcxSideBarPainter;
    FPressedGroup: TcxSideBarGroup;
    FPressedLink: TcxSideBarLink;
    FSaveCursor: TCursor;
    FScrollDown: TcxSideBarScroll;
    FScrollTimer: TTimer;
    FScrollUp: TcxSideBarScroll;
    FSelectedGroup: TcxSideBarGroup;
    FSelectedLink: TcxSideBarLink;
    FSmallImages: TImageList;
    FSmallImagesChangeLink: TChangeLink;
    FStyle: TcxSideBarStyle;
    FTopLink: TcxSideBarLink;
    FUpdateLockCount: Integer;
    FViewInfo: TcxSideBarViewInfo;
    FVisibleGroups: TList;
    FWaitingGroup: TcxSideBarGroup;
    FWaitingTimer: TTimer;

    function GetGroup(Index: Integer): TcxSideBarGroup;
    function GetGroupCount: Integer;
    function GetInternalDragging: Boolean;
    function GetItem(Index: Integer): TcxSideBarItem;
    function GetItemCount: Integer;
    function GetLinkInsertionPosLink: TcxSideBarLink;
    function GetStayPressed: Boolean;
    function GetTopLinkIndex: Integer;
    function GetVisibleGroup(Index: Integer): TcxSideBarGroup;
    function GetVisibleGroupCount: Integer;
    procedure SetActiveGroup(Value: TcxSideBarGroup);
    procedure SetActiveItem(Value: TcxSideBarItem);
    procedure SetAssignedValues(Value: TcxSideBarAssignedValues);
    procedure SetBackground(Value: TcxSideBarBackground);
    procedure SetGroupFont(Value: TFont);
    procedure SetLargeImages(Value: TImageList);
    procedure SetLinkInsertionPos(Value: Integer);
    procedure SetLookAndFeel(Value: TcxSideBarLookAndFeel);
    procedure SetOptions(Value: TcxSideBarOptions);
    procedure SetPressedGroup(Value: TcxSideBarGroup);
    procedure SetPressedLink(Value: TcxSideBarLink);
    procedure SetSelectedGroup(Value: TcxSideBarGroup);
    procedure SetSelectedLink(Value: TcxSideBarLink);
    procedure SetSmallImages(Value: TImageList);
    procedure SetStyle(Value: TcxSideBarStyle);
    procedure SetTopLink(Value: TcxSideBarLink);
    procedure SetTopLinkIndex(Value: Integer);
    procedure SetWaitingGroup(Value: TcxSideBarGroup);

    procedure SyncGroupFont;
    procedure DestroyGroups;
    procedure DestroyItems;
    procedure GroupFontChangeHandler(Sender: TObject);
    procedure ImagesChanged(Sender: TObject);
    function IsGroupFontAssigned: Boolean;
    procedure MoveGroup(FromIndex, ToIndex: Integer);
    procedure MoveItem(FromIndex, ToIndex: Integer);
    procedure ScrollTimerHandler(Sender: TObject);
    procedure WaitingTimerHandler(Sender: TObject);
  protected
    function CanDrag(X, Y: Integer): Boolean; override;
    procedure DoEndDrag(Target: TObject; X, Y: Integer); override;
    procedure DragOver(Source: TObject; X, Y: Integer; State: TDragState;
      var Accept: Boolean); override;
    procedure FontChanged; override;
    procedure GetChildren(Proc: TGetChildProc; Root: TComponent); override;
    function GetDesignHitTest(X, Y: Integer; Shift: TShiftState): Boolean; override;
    function MayFocus: Boolean; override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure MouseLeave(AControl: TControl); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState;
      X, Y: Integer); override;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    procedure Paint; override;
    procedure Resize; override;
    procedure SetName(const NewName: TComponentName); override;
    function StartDrag(var DragObject: TDragObject): Boolean; override;

    procedure ActiveGroupChanged; virtual;
    procedure ActiveItemChanged(PrevActiveItem: TcxSideBarItem); virtual;
    procedure Changed(ViewInfoChanged: Boolean);
    procedure CheckTopLink;
    procedure DoGroupCreated(AGroup: TcxSideBarGroup);
    procedure DoGroupDeleted(AGroup: TcxSideBarGroup);
    procedure DoItemCreated(AItem: TcxSideBarItem);
    procedure DoItemDeleted(AItem: TcxSideBarItem);
    function FindActiveGroup: TcxSideBarGroup;
    function FindTopLink: TcxSideBarLink;
    function GetLinkForLinkInsertionPos(Pos: Integer): TcxSideBarLink;
    function GetLinkInsertionPosForLink(Link: TcxSideBarLink): Integer;
    function GetPainterClass: TcxSideBarPainterClass; virtual;
    procedure GroupFontChanged;
    procedure RefreshVisibleGroups;

    function GetScrollByScrollTag(ScrollTag: Integer): TcxSideBarScroll;
    procedure StartScrollTimer(ScrollTag: Integer; IsDragDrop: Boolean);
    procedure StopScrollTimer;

    procedure SetCursor(ACursor: TCursor);
    procedure RestoreCursor;

    property Canvas: TcxCanvas read FCanvas;
    property InplaceEdit: TcxSideBarInplaceEdit read FInplaceEdit;
    property Painter: TcxSideBarPainter read FPainter;
    property PressedGroup: TcxSideBarGroup read FPressedGroup write SetPressedGroup;
    property PressedLink: TcxSideBarLink read FPressedLink write SetPressedLink;
    property ScrollDown: TcxSideBarScroll read FScrollDown;
    property ScrollUp: TcxSideBarScroll read FScrollUp;
    property SelectedGroup: TcxSideBarGroup read FSelectedGroup write SetSelectedGroup;
    property SelectedLink: TcxSideBarLink read FSelectedLink write SetSelectedLink;
    property ViewInfo: TcxSideBarViewInfo read FViewInfo;
    property WaitingGroup: TcxSideBarGroup read FWaitingGroup write SetWaitingGroup;

    property DraggingLink: TcxSideBarLink read FDraggingLink write FDraggingLink;
    property InternalDragging: Boolean read GetInternalDragging;
    property LastPressedLink: TcxSideBarLink read FLastPressedLink write FLastPressedLink;
    property LinkInsertionPos: Integer read FLinkInsertionPos write SetLinkInsertionPos;
    property LinkInsertionPosLink: TcxSideBarLink read GetLinkInsertionPosLink;

    property StayPressed: Boolean read GetStayPressed;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure DragDrop(Source: TObject; X, Y: Integer); override;

    function AddGroup: TcxSideBarGroup;
    function AddItem: TcxSideBarItem;
    procedure BeginUpdate;
    procedure CancelUpdate;
    procedure EndUpdate;

    procedure StartEditing(AEditingObject: TObject);
    procedure FinishEditing(Accept: Boolean);

    property GroupCount: Integer read GetGroupCount;
    property Groups[Index: Integer]: TcxSideBarGroup read GetGroup;
    property ItemCount: Integer read GetItemCount;
    property Items[Index: Integer]: TcxSideBarItem read GetItem;
    property TopLink: TcxSideBarLink read FTopLink write SetTopLink;
    property TopLinkIndex: Integer read GetTopLinkIndex write SetTopLinkIndex;
    property VisibleGroupCount: Integer read GetVisibleGroupCount;
    property VisibleGroups[Index: Integer]: TcxSideBarGroup read GetVisibleGroup;
  published
    property ActiveGroup: TcxSideBarGroup read FActiveGroup write SetActiveGroup;
    property ActiveItem: TcxSideBarItem read FActiveItem write SetActiveItem;
    property Align;
    property Anchors;
    property AssignedValues: TcxSideBarAssignedValues read FAssignedValues
      write SetAssignedValues stored False;
    property Background: TcxSideBarBackground read FBackground write SetBackground;
    property Color default {$IFDEF LINUX}clGray{$ELSE}clBtnShadow{$ENDIF};
    property Constraints;
    property DragMode;
    property Enabled;
    property Font;
    property GroupFont: TFont read FGroupFont write SetGroupFont stored IsGroupFontAssigned;
    property LargeImages: TImageList read FLargeImages write SetLargeImages;
    property LookAndFeel: TcxSideBarLookAndFeel read FLookAndFeel write SetLookAndFeel
      default sblfStandard;
    property Options: TcxSideBarOptions read FOptions write SetOptions default [];
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property PopupMenu;
    property ShowHint;
    property SmallImages: TImageList read FSmallImages write SetSmallImages;
    property Style: TcxSideBarStyle read FStyle write SetStyle default ssLargeImages;
    property Visible;

    property OnClick;
    property OnContextPopup;
    property OnDblClick;
    property OnDragDrop;
    property OnDragOver;
    property OnEndDrag;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnMouseWheelDown;
    property OnMouseWheelUp;
    property OnStartDrag;
  end;

var
  cxSideBarDesignerLink: TcxSideBarDesignerLink;

implementation

uses
  Math;

const
  DefaultLargeImageSize = 32;
  DefaultSmallImageSize = 16;
  GroupTextOffset = 3;
  ImageBorderSize = 1;
  ImageOffset = 2;
  LinkOffsetX = 6;
  LinkOffsetY = 8;
  ScrollOffset = 5;
  ScrollSize = 16;
  ScrollArrowHeight = ScrollSize div 4;
  ScrollArrowWidth = 2 * ScrollArrowHeight - 1;
  TextOffsetForLargeImages = 2;
  TextOffsetForSmallImages = 2;

  LinkHitTestCodes = [htLinkImage, htLinkCaption];

  ScrollTimeInterval = 250;
  ScrollTimeDragDropInterval = 100;
  ScrollUpTag = 1;
  ScrollDownTag = 2;

  WaitingInterval = 400;

{ TcxSideBarDesignerLink }

function TcxSideBarDesignerLink.GetDesigner(AControl: TcxSideBar): TDesigner;
begin
  if GetParentForm(AControl) <> nil then
    Result := GetParentForm(AControl).{$IFDEF VCL}Designer{$ELSE}DesignerHook{$ENDIF}
  else
    Result := nil;
end;

procedure TcxSideBarDesignerLink.ControlNameChanged(AControl: TcxSideBar);
begin
end;

procedure TcxSideBarDesignerLink.GroupNameChanged(AControl: TcxSideBar);
begin
end;

procedure TcxSideBarDesignerLink.ItemNameChanged(AControl: TcxSideBar);
begin
end;

procedure TcxSideBarDesignerLink.LinkItemChanged(AControl: TcxSideBar);
begin
end;

procedure TcxSideBarDesignerLink.Modified(AControl: TcxSideBar);
begin
  if not AControl.IsLoading and (GetDesigner(AControl) <> nil) then
    GetDesigner(AControl).Modified;
end;

procedure TcxSideBarDesignerLink.SetSelection(AControl: TcxSideBar; AObject: TPersistent);
begin
end;

procedure TcxSideBarDesignerLink.ShowDefaultEventHandler(AItem: TcxSideBarItem);
begin
end;

{ TcxSideBarItem }

constructor TcxSideBarItem.Create(AOwner: TComponent);
begin
  inherited;
  FEnabled := True;
  FLargeImageIndex := -1;
  FLinks := TList.Create;
  FSmallImageIndex := -1;
  FVisible := True;
end;

destructor TcxSideBarItem.Destroy;
begin
  Control := nil;
  DestroyLinks;
  FLinks.Free;
  inherited;
end;

function TcxSideBarItem.GetActive: Boolean;
begin
  Result := Control.ActiveItem = Self;
end;

function TcxSideBarItem.GetIndex: Integer;
begin
  if FControl = nil then
    Result := -1
  else
    Result := FControl.FItems.IndexOf(Self);
end;

function TcxSideBarItem.GetLink(Index: Integer): TcxSideBarLink;
begin
  Result := TcxSideBarLink(FLinks[Index]);
end;

function TcxSideBarItem.GetLinkCount: Integer;
begin
  Result := FLinks.Count;
end;

function TcxSideBarItem.GetVisible: Boolean;
begin
  Result := FVisible and (Index <> -1);
end;

procedure TcxSideBarItem.SetActive(Value: Boolean);
begin
  if Value then
    Control.ActiveItem := Self;
end;

procedure TcxSideBarItem.SetCaption(const Value: string);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Changed(True);
  end;
end;

procedure TcxSideBarItem.SetControl(Value: TcxSideBar);
begin
  if FControl <> Value then
  begin
    if FControl <> nil then FControl.DoItemDeleted(Self);
    FControl := Value;
    if FControl <> nil then FControl.DoItemCreated(Self);
  end;
end;

procedure TcxSideBarItem.SetEnabled(Value: Boolean);
begin
  if FEnabled <> Value then
  begin
    FEnabled := Value;
    Changed(True);
  end;
end;

procedure TcxSideBarItem.SetIndex(Value: Integer);
begin
  if Index <> Value then
  begin
    Control.MoveItem(Index, Value);
  end;
end;

procedure TcxSideBarItem.SetLargeImageIndex(Value: Integer);
begin
  if FLargeImageIndex <> Value then
  begin
    FLargeImageIndex := Value;
    Changed(False);
  end;
end;

procedure TcxSideBarItem.SetSmallImageIndex(Value: Integer);
begin
  if FSmallImageIndex <> Value then
  begin
    FSmallImageIndex := Value;
    Changed(False);
  end;
end;

procedure TcxSideBarItem.SetVisible(Value: Boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    VisibleChanged;
  end;
end;

procedure TcxSideBarItem.SetName(const NewName: TComponentName);
begin
  inherited;
  cxSideBarDesignerLink.ItemNameChanged(Control);
end;

procedure TcxSideBarItem.SetParentComponent(Value: TComponent);
begin
  if Value is TcxSideBar then
    Control := TcxSideBar(Value);
end;

procedure TcxSideBarItem.AddLink(ALink: TcxSideBarLink);
begin
  FLinks.Add(ALink);
end;

procedure TcxSideBarItem.DestroyLinks;
begin
  while LinkCount <> 0 do
    Links[0].Free;
end;

procedure TcxSideBarItem.RemoveLink(ALink: TcxSideBarLink);
begin
  FLinks.Remove(ALink);
end;

procedure TcxSideBarItem.ActiveChanged;
begin
  Changed(False);
  if Active then DoClick;
end;

procedure TcxSideBarItem.Changed(ViewInfoChanged: Boolean);
var
  I: Integer;
begin
  for I := 0 to LinkCount - 1 do
    Links[I].Changed(ViewInfoChanged);
end;

procedure TcxSideBarItem.DoClick;
begin
  if Assigned(FOnClick) then FOnClick(Self);
end;

procedure TcxSideBarItem.VisibleChanged;
var
  I: Integer;
begin
  for I := 0 to LinkCount - 1 do
    Links[I].VisibleChanged;
end;

function TcxSideBarItem.GetParentComponent: TComponent;
begin
  Result := Control;
end;

function TcxSideBarItem.HasParent: Boolean;
begin
  Result := Control <> nil;
end;

{ TcxSideBarLink }

constructor TcxSideBarLink.Create(Collection: TCollection);
begin
  inherited;
end;

destructor TcxSideBarLink.Destroy;
begin
  Item := nil;
  Links.DoLinkDeleted(Self);
  VisibleChanged;
  inherited;
end;

function TcxSideBarLink.GetActive: Boolean;
begin
  Result := (Item <> nil) and Item.Active;
end;

function TcxSideBarLink.GetCaption: string;
begin
  if IsCaptionAssigned then
    Result := FCaption
  else
    if Item = nil then
      Result := ''
    else
      Result := Item.Caption;
end;

function TcxSideBarLink.GetControl: TcxSideBar;
begin
  Result := Links.Control;
end;

function TcxSideBarLink.GetEnabled: Boolean;
begin
  Result := Item.Enabled;
end;

function TcxSideBarLink.GetGroup: TcxSideBarGroup;
begin
  Result := Links.Group;
end;

function TcxSideBarLink.GetImageIndex: Integer;
begin
  if Group.Style = ssLargeImages then
    Result := Item.LargeImageIndex
  else
    Result := Item.SmallImageIndex;
end;

function TcxSideBarLink.GetImages: TImageList;
begin
  if Group.Style = ssLargeImages then
    Result := Control.LargeImages
  else
    Result := Control.SmallImages
end;

function TcxSideBarLink.GetLinks: TcxSideBarLinks;
begin
  Result := TcxSideBarLinks(Collection);
end;

function TcxSideBarLink.GetPressed: Boolean;
begin
  Result :=
    (Control.PressedLink = Self) or
    Active and Control.StayPressed;
end;

function TcxSideBarLink.GetSelected: Boolean;
begin
  Result := Control.SelectedLink = Self;
end;

function TcxSideBarLink.GetViewState: TcxSideBarViewState;
begin
  if Pressed then
    Result := vsPressed
  else
    if Selected then
      Result := vsSelected
    else
      Result := vsNone;
end;

function TcxSideBarLink.GetVisible: Boolean;
begin
  Result := (Item <> nil) and Item.Visible;
end;

function TcxSideBarLink.GetVisibleIndex: Integer;
begin
  Result := Links.FVisibleItems.IndexOf(Self);
end;

function TcxSideBarLink.GetVisuallyVisible: Boolean;
begin
  Result := Group.Active and
    (Control.TopLinkIndex <= VisibleIndex) and
    (VisibleIndex < Control.TopLinkIndex + Control.ViewInfo.VisibleLinkCount);
end;

procedure TcxSideBarLink.SetAssignedValues(Value: TcxSideBarLinkAssignedValues);
var
  ChangedValues: TcxSideBarLinkAssignedValues;
begin
  if FAssignedValues <> Value then
  begin
    Byte(ChangedValues) := Byte(FAssignedValues) xor Byte(Value);
    FAssignedValues := Value;
    if avCaption in ChangedValues then
      CaptionChanged;
  end;
end;

procedure TcxSideBarLink.SetCaption(const Value: string);
begin
  if Caption <> Value then
  begin
    FCaption := Value;
    if IsCaptionAssigned then
      CaptionChanged
    else
      AssignedValues := AssignedValues + [avCaption];
  end;
end;

procedure TcxSideBarLink.SetItem(Value: TcxSideBarItem);
begin
  if FItem <> Value then
  begin
    if FItem <> nil then FItem.RemoveLink(Self);
    FItem := Value;
    if FItem <> nil then FItem.AddLink(Self);
    VisibleChanged;
    cxSideBarDesignerLink.LinkItemChanged(Control);
  end;
end;

procedure TcxSideBarLink.SetVisibleIndex(Value: Integer);
begin
  if (VisibleIndex <> Value) and
    (0 <= Value) and (Value < Links.VisibleCount) then
    Index := Links.VisibleItems[Value].Index;
end;

function TcxSideBarLink.IsCaptionAssigned: Boolean;
begin
  Result := avCaption in AssignedValues;
end;

procedure TcxSideBarLink.CaptionChanged;
begin
  Changed(True);
end;

procedure TcxSideBarLink.Changed(ViewInfoChanged: Boolean);
begin
  if VisuallyVisible then
    with Control.Painter do
      if ViewInfoChanged then
        Invalidate(nil, nil, htBackground)
      else
      begin
        Invalidate(nil, Self, htLinkImage);
        Invalidate(nil, Self, htLinkCaption);
      end;
end;

procedure TcxSideBarLink.PressedChanged;
begin
  ViewStateChanged;
end;

procedure TcxSideBarLink.SelectedChanged;
begin
  if not Pressed then ViewStateChanged;
end;

procedure TcxSideBarLink.ViewStateChanged;
begin
  Control.Painter.InvalidateLinkImageBorder(Self);
end;

procedure TcxSideBarLink.VisibleChanged;
begin
  Links.VisibleItemsChanged;
  if not Visible then
  begin
    if Pressed then
      Control.PressedLink := nil;
    if Selected then
      Control.SelectedLink := nil;
    if Control.TopLink = Self then
      Control.TopLink := Control.FindTopLink;
    if Control.InternalDragging and (Control.DraggingLink = Self) then
      Control.DraggingLink := nil;
  end
  else
    if Group.Active and (Control.TopLink = nil) then
      Control.TopLink := Control.FindTopLink;
end;

procedure TcxSideBarLink.Assign(Source: TPersistent);
var
  Link: TcxSideBarLink;
begin
  if Source is TcxSideBarLink then
  begin
    Link := TcxSideBarLink(Source);
    Item := Link.Item;
    if Link.IsCaptionAssigned then
      Caption := Link.Caption
    else
      AssignedValues := AssignedValues - [avCaption];
  end
  else
    inherited;
end;

procedure TcxSideBarLink.MakeVisuallyVisible;
begin
  if not Visible then Exit;
  Group.Active := True;
  Control.ViewInfo.CalcViewInfo;
  if not VisuallyVisible then
    if VisibleIndex < Control.TopLinkIndex then
      Control.TopLink := Self
    else
      repeat
        with Control do
        begin
          TopLinkIndex := TopLinkIndex + 1;
          ViewInfo.CalcViewInfo;
        end;  
      until VisuallyVisible;
end;

procedure TcxSideBarLink.StartEditing;
begin
  Control.StartEditing(Self);
end;

{ TcxSideBarLinks }

constructor TcxSideBarLinks.Create(AGroup: TcxSideBarGroup);
begin
  inherited Create(TcxSideBarLink);
  FGroup := AGroup;
  FVisibleItems := TList.Create;
end;

destructor TcxSideBarLinks.Destroy;
begin
  FVisibleItems.Free;
  FVisibleItems := nil;
  inherited;
end;

function TcxSideBarLinks.GetControl: TcxSideBar;
begin
  Result := Group.Control;
end;

function TcxSideBarLinks.GetItem(Index: Integer): TcxSideBarLink;
begin
  Result := TcxSideBarLink(inherited GetItem(Index));
end;

function TcxSideBarLinks.GetVisibleCount: Integer;
begin
  if FVisibleItems = nil then
    Result := 0
  else
    Result := FVisibleItems.Count;
end;

function TcxSideBarLinks.GetVisibleItem(Index: Integer): TcxSideBarLink;
begin
  Result := TcxSideBarLink(FVisibleItems[Index]);
end;

procedure TcxSideBarLinks.SetItem(Index: Integer; Value: TcxSideBarLink);
begin
  inherited SetItem(Index, Value);
end;

function TcxSideBarLinks.GetOwner: TPersistent;
begin
  Result := Group;
end;

procedure TcxSideBarLinks.Update(Item: TCollectionItem);
begin
  inherited;
  VisibleItemsChanged;
end;

procedure TcxSideBarLinks.DoLinkDeleted(ALink: TcxSideBarLink);
begin
end;

procedure TcxSideBarLinks.RefreshVisibleItems;
var
  I: Integer;
begin
  if FVisibleItems = nil then Exit;
  FVisibleItems.Clear;
  for I := 0 to Count - 1 do
    if Items[I].Visible then
      FVisibleItems.Add(Items[I]);
end;

procedure TcxSideBarLinks.VisibleItemsChanged;
begin
  RefreshVisibleItems;
  if Group.Active then Group.Changed(True);
end;

function TcxSideBarLinks.Add: TcxSideBarLink;
begin
  Result := TcxSideBarLink(inherited Add);
end;

{ TcxSideBarGroup }

constructor TcxSideBarGroup.Create(AOwner: TComponent);
begin
  inherited;
  FLinks := TcxSideBarLinks.Create(Self);
  FVisible := True;
end;

destructor TcxSideBarGroup.Destroy;
begin
  FLinks.Free;
  Control := nil;
  inherited;
end;

function TcxSideBarGroup.GetActive: Boolean;
begin
  Result := Control.ActiveGroup = Self;
end;

function TcxSideBarGroup.GetIndex: Integer;
begin
  Result := Control.FGroups.IndexOf(Self);
end;

function TcxSideBarGroup.GetPressed: Boolean;
begin
  Result := Control.PressedGroup = Self;
end;

function TcxSideBarGroup.GetSelected: Boolean;
begin
  Result := Control.SelectedGroup = Self;
end;

function TcxSideBarGroup.GetStyle: TcxSideBarStyle;
begin
  if avStyle in AssignedValues then
    Result := FStyle
  else
    Result := FControl.Style;
end;

function TcxSideBarGroup.GetViewState: TcxSideBarViewState;
begin
  if Pressed then
    Result := vsPressed
  else
    if Selected then
      Result := vsSelected
    else
      Result := vsNone;
end;

function TcxSideBarGroup.GetVisible: Boolean;
begin
  Result := FVisible and (Index <> -1);
end;

function TcxSideBarGroup.GetVisibleIndex: Integer;
begin
  Result := Control.FVisibleGroups.IndexOf(Self);
end;

function TcxSideBarGroup.GetVisuallyVisible: Boolean;
begin
  Result :=
    (0 <= VisibleIndex) and (VisibleIndex < Control.ViewInfo.StoredGroupCount);
end;

procedure TcxSideBarGroup.SetActive(Value: Boolean);
begin
  if Value then
    Control.ActiveGroup := Self;
end;

procedure TcxSideBarGroup.SetAssignedValues(Value: TcxSideBarGroupAssignedValues);
var
  ChangedValues: TcxSideBarGroupAssignedValues;
begin
  if FAssignedValues <> Value then
  begin
    Byte(ChangedValues) := Byte(FAssignedValues) xor Byte(Value);
    FAssignedValues := Value;
    if avStyle in ChangedValues then
      StyleChanged;
  end;
end;

procedure TcxSideBarGroup.SetCaption(const Value: string);
begin
  if FCaption <> Value then
  begin
    FCaption := Value;
    Changed(False);
  end;
end;

procedure TcxSideBarGroup.SetControl(Value: TcxSideBar);
begin
  if FControl <> Value then
  begin
    if FControl <> nil then FControl.DoGroupDeleted(Self);
    FControl := Value;
    if FControl <> nil then FControl.DoGroupCreated(Self);
  end;
end;

procedure TcxSideBarGroup.SetIndex(Value: Integer);
begin
  if Index <> Value then
  begin
    Control.MoveGroup(Index, Value);
    VisibleChanged;
  end;
end;

procedure TcxSideBarGroup.SetLinks(Value: TcxSideBarLinks);
begin
  FLinks.Assign(Value);
end;

procedure TcxSideBarGroup.SetStyle(Value: TcxSideBarStyle);
begin
  if FStyle <> Value then
  begin
    FStyle := Value;
    if IsStyleAssigned then
      StyleChanged
    else
      AssignedValues := AssignedValues + [avStyle];
  end;
end;

procedure TcxSideBarGroup.SetVisible(Value: Boolean);
begin
  if FVisible <> Value then
  begin
    FVisible := Value;
    VisibleChanged;
  end;
end;

function TcxSideBarGroup.IsStyleAssigned: Boolean;
begin
  Result := avStyle in AssignedValues;
end;

procedure TcxSideBarGroup.SetName(const NewName: TComponentName);
begin
  inherited;
  cxSideBarDesignerLink.GroupNameChanged(Control);
end;

procedure TcxSideBarGroup.SetParentComponent(Value: TComponent);
begin
  if Value is TcxSideBar then
    Control := TcxSideBar(Value);
end;

procedure TcxSideBarGroup.Changed(ViewInfoChanged: Boolean);
begin
  with Control.Painter do
    if ViewInfoChanged then
      Invalidate(nil, nil, htNone)
    else
      Invalidate(Self, nil, htGroup);
end;

procedure TcxSideBarGroup.PressedChanged;
begin
  ViewStateChanged;
end;

procedure TcxSideBarGroup.SelectedChanged;
begin
  ViewStateChanged;
end;

procedure TcxSideBarGroup.StyleChanged;
begin
  Changed(True);
end;

procedure TcxSideBarGroup.ViewStateChanged;
begin
  Changed(False);
end;

procedure TcxSideBarGroup.VisibleChanged;
begin
  Control.RefreshVisibleGroups;
  if not Visible then
  begin
    if Selected then
      Control.SelectedGroup := nil;
    if Pressed then
      Control.PressedGroup := nil;
  end;
  if Visible and (Control.ActiveGroup = nil) or not Visible and Active then
    Control.ActiveGroup := Control.FindActiveGroup;
  Changed(True);
end;

function TcxSideBarGroup.GetParentComponent: TComponent;
begin
  Result := Control;
end;

function TcxSideBarGroup.HasParent: Boolean;
begin
  Result := Control <> nil;
end;

function TcxSideBarGroup.AddLink(AItem: TcxSideBarItem): TcxSideBarLink;
begin
  Result := Links.Add;
  Result.Item := AItem;
end;

procedure TcxSideBarGroup.StartEditing;
begin
  Control.StartEditing(Self);
end;

{ TcxSideBarScroll }

constructor TcxSideBarScroll.Create(AControl: TcxSideBar);
begin
  inherited Create;
  FControl := AControl;
end;

procedure TcxSideBarScroll.SetViewState(Value: TcxSideBarViewState);
begin
  if FViewState <> Value then
  begin
    FViewState := Value;
    Changed(False);
    Control.StopScrollTimer;
  end;
end;

procedure TcxSideBarScroll.Changed(ViewInfoChanged: Boolean);
begin
  Control.Painter.Invalidate(nil, nil, GetHitTestCode);
end;

{ TcxSideBarScrollDown }

procedure TcxSideBarScrollDown.DoClick;
begin
  with Control do
    TopLinkIndex := TopLinkIndex + 1;
end;

function TcxSideBarScrollDown.GetHitTestCode: TcxSideBarHitTestCode;
begin
  Result := htScrollDown;
end;

function TcxSideBarScrollDown.GetIsScrollUp: Boolean;
begin
  Result := False;
end;

function TcxSideBarScrollDown.GetVisible: Boolean;
begin
  Result := Control.ViewInfo.CanScrollDown;
end;

{ TcxSideBarScrollUp }

procedure TcxSideBarScrollUp.DoClick;
begin
  with Control do
    TopLinkIndex := TopLinkIndex - 1;
end;

function TcxSideBarScrollUp.GetHitTestCode: TcxSideBarHitTestCode;
begin
  Result := htScrollUp;
end;

function TcxSideBarScrollUp.GetIsScrollUp: Boolean;
begin
  Result := True;
end;

function TcxSideBarScrollUp.GetVisible: Boolean;
begin
  Result := Control.ViewInfo.CanScrollUp;
end;

{ TcxSideBarBackground }

constructor TcxSideBarBackground.Create(AControl: TcxSideBar);
begin
  inherited Create;
  FBeginColor := clBlue;
  FControl := AControl;
  FEndColor := clBlack;
  FMiddleColor := clNone;
  FTexture := TBitmap.Create;
  FTexture.OnChange := TextureChanged;
end;

destructor TcxSideBarBackground.Destroy;
begin
  FTexture.Free;
  inherited;
end;

procedure TcxSideBarBackground.SetBeginColor(Value: TColor);
begin
  if FBeginColor <> Value then
  begin
    FBeginColor := Value;
    Changed;
  end;
end;

procedure TcxSideBarBackground.SetEndColor(Value: TColor);
begin
  if FEndColor <> Value then
  begin
    FEndColor := Value;
    Changed;
  end;
end;

procedure TcxSideBarBackground.SetMiddleColor(Value: TColor);
begin
  if FMiddleColor <> Value then
  begin
    FMiddleColor := Value;
    Changed;
  end;
end;

procedure TcxSideBarBackground.SetMode(Value: TcxSideBarBackgroundMode);
begin
  if FMode <> Value then
  begin
    FMode := Value;
    Changed;
  end;
end;

procedure TcxSideBarBackground.SetTexture(Value: TBitmap);
begin
  FTexture.Assign(Value);
end;

procedure TcxSideBarBackground.TextureChanged(Sender: TObject);
begin
  Changed;
end;

procedure TcxSideBarBackground.Changed;
begin
  Control.Changed(False);
end;

procedure TcxSideBarBackground.Assign(Source: TPersistent);
var
  Background: TcxSideBarBackground;
begin
  if Source is TcxSideBarBackground then
  begin
    Background := TcxSideBarBackground(Source);
    BeginColor := Background.BeginColor;
    EndColor := Background.EndColor;
    MiddleColor := Background.MiddleColor;
    Mode := Background.Mode;
    Texture := Background.Texture;
  end
  else
    inherited;
end;

{ TcxSideBarEditLink }

constructor TcxSideBarEditLink.Create(AInplaceEdit: TcxSideBarInplaceEdit;
  AEditingObject: TObject);
begin
  inherited Create;
  FInplaceEdit := AInplaceEdit;
end;

{ TcxSideBarEditLinkForLink }

constructor TcxSideBarEditLinkForLink.Create(AInplaceEdit: TcxSideBarInplaceEdit;
  AEditingObject: TObject);
begin
  inherited;
  FLink := TcxSideBarLink(AEditingObject);
end;

procedure TcxSideBarEditLinkForLink.Initiate;
const
  Alignments: array[TcxSideBarStyle] of TAlignment = (taCenter, taLeftJustify);
begin
  with InplaceEdit do
  begin
    Alignment := Alignments[Link.Group.Style];
    Font := Control.Font;
    Font.Color := clWindowText;
    Expandable := True;
    OriginalBounds := Control.ViewInfo.GetRect(nil, Link, htLinkCaption);
    Text := Link.Caption;
  end;  
end;

procedure TcxSideBarEditLinkForLink.StoreValue;
begin
  Link.Caption := InplaceEdit.Text;
end;

{ TcxSideBarEditLinkForGroup }

constructor TcxSideBarEditLinkForGroup.Create(AInplaceEdit: TcxSideBarInplaceEdit;
  AEditingObject: TObject);
begin
  inherited;
  FGroup := TcxSideBarGroup(AEditingObject);
end;

procedure TcxSideBarEditLinkForGroup.Initiate;
begin
  with InplaceEdit do
  begin
    Alignment := taLeftJustify;//Center;
    Font := Control.GroupFont;
    Font.Color := clWindowText;
    Expandable := False;
    OriginalBounds := Control.ViewInfo.GetRect(Group, nil, htGroup);
    Text := Group.Caption;
  end;  
end;

procedure TcxSideBarEditLinkForGroup.StoreValue;
begin
  Group.Caption := InplaceEdit.Text;
end;

{ TcxSideBarInplaceEdit }

constructor TcxSideBarInplaceEdit.Create(AControl: TcxSideBar);
begin
  inherited Create(nil);
  FControl := AControl;
{$IFNDEF VCL}
  BorderStyle := bsSingle;
  ScrollBars := ssNone;
{$ENDIF}
  Visible := False;
  { doesn't exists now in D6  !!!!!!!!!!!!!!!!!!!!!!
  WrapBreak := wbAnywhere;
  }
  OnChange := ChangeHandler;  //!!!
  FExpandable := True;
end;

function TcxSideBarInplaceEdit.GetBounds: TRect;
const
  Alignments: array[Boolean] of Integer = (cxAlignLeft, cxAlignHCenter);

  function TextAreaOffset: TPoint;
  {$IFDEF VCL}
  var
    R: TRect;
  begin
    if HandleAllocated then
    begin
      SendMessage(Handle, EM_GETRECT, 0, LPARAM(@R));
      TextAreaOffset := R.TopLeft;
    end
    else
      TextAreaOffset := Point(5, 2);
  end;
  {$ELSE}
  begin
    TextAreaOffset := Point(1, 1);
  end;
  {$ENDIF}

begin
  if Expandable then
  begin
    Result := Control.ViewInfo.LinksAreaBounds;
    if OriginalBounds.Top < Result.Top then
      Result := Rect(0, 0, 0, 0)
    else
    begin
      if Alignment = taLeftJustify then
        Result.Left := OriginalBounds.Left;
      Result.Top := OriginalBounds.Top;
      Result.Bottom := Result.Top;

      Control.Canvas.TextExtent(Text, Result,
        Alignments[Alignment = taCenter] or cxAlignTop or cxWordBreak);

      with TextAreaOffset do
        InflateRect(Result, X, Y);
      with Control.ViewInfo.LinksAreaBounds do
        if Result.Bottom > Bottom then
          Result.Bottom := Bottom;
    end;
  end
  else
    Result := OriginalBounds;
end;

procedure TcxSideBarInplaceEdit.SetEditingObject(Value: TObject);
begin
  if Value = nil then
  begin
    FEditLink.Free;
    FEditLink := nil;
  end
  else
  begin
    if Value is TcxSideBarLink then
      FEditLink := TcxSideBarEditLinkForLink.Create(Self, Value)
    else
      FEditLink := TcxSideBarEditLinkForGroup.Create(Self, Value);
    FEditLink.Initiate;  
  end;
end;

procedure TcxSideBarInplaceEdit.SetExpandable(Value: Boolean);
begin
  if FExpandable <> Value then
  begin
    FExpandable := Value;
    WordWrap := Value;
  end;
end;

procedure TcxSideBarInplaceEdit.ChangeHandler(Sender: TObject);
begin
  //
end;

{$IFDEF VCL}
procedure TcxSideBarInplaceEdit.WMKillFocus(var Message: TMessage);
begin
  inherited;
  DoFocusOut;
end;
{$ENDIF}

procedure TcxSideBarInplaceEdit.Change;
begin
  inherited;
  BoundsRect := Bounds;
end;

{$IFNDEF VCL}
function TcxSideBarInplaceEdit.EventFilter(Sender: QObjectH; Event: QEventH): Boolean;
begin
  Result := inherited EventFilter(Sender, Event);
  case QEvent_type(Event) of
    QEventType_FocusOut:
      DoFocusOut;
  end;
end;
{$ENDIF}

procedure TcxSideBarInplaceEdit.KeyDown(var Key: Word; Shift: TShiftState);
begin
  inherited;
  case TranslateKey(Key) of
    VK_ESCAPE:
      FinishEditing(False);
    VK_RETURN:
      FinishEditing(True);
  end;
end;

procedure TcxSideBarInplaceEdit.DoFocusOut;
begin
  FinishEditing(False);
end;

procedure TcxSideBarInplaceEdit.StartEditing;
begin
{$IFNDEF VCL}
  QMultiLineEdit_setWrapPolicy(Handle, QMultiLineEditWrapPolicy_Anywhere); //!!! use WrapBreak
{$ENDIF}
  Parent := Control;
{$IFDEF VCL}
  Ctl3d := False;
{$ENDIF}  
  BoundsRect := Bounds;
  Visible := True;
end;

procedure TcxSideBarInplaceEdit.FinishEditing(Accept: Boolean);
begin
  if Accept and (EditLink <> nil) then
    EditLink.StoreValue;
  EditingObject := nil;
  Visible := False;
end;

{ TcxSideBarGroupViewInfo }

constructor TcxSideBarGroupViewInfo.Create(AViewInfo: TcxSideBarViewInfo);
begin
  inherited Create;
  FViewInfo := AViewInfo;
end;

function TcxSideBarGroupViewInfo.WantsMouse(const P: TPoint): Boolean;
begin
  Result := PtInRect(Bounds, P);
end;

{ TcxSideBarLinkViewInfo }

constructor TcxSideBarLinkViewInfo.Create(AViewInfo: TcxSideBarViewInfo);
begin
  inherited Create;
  FViewInfo := AViewInfo;
end;

function TcxSideBarLinkViewInfo.WantsMouse(const P: TPoint;
  var OverImage: Boolean): Boolean;
begin
  OverImage := PtInRect(ImageBounds, P);
  Result := OverImage or PtInRect(CaptionBoundsForMouse, P);
end;

{ TcxSideBarViewInfo }

constructor TcxSideBarViewInfo.Create(AControl: TcxSideBar);
begin
  inherited Create;
  FControl := AControl;
end;

destructor TcxSideBarViewInfo.Destroy;
begin
  DestroyViewInfo;
  inherited;
end;

function TcxSideBarViewInfo.GetBorderSize: Integer;
begin
  if Flat then
    Result := 1
  else
    Result := 2;
end;

function TcxSideBarViewInfo.GetBounds: TRect;
begin
  with Control do
    Result := Rect(0, 0, Width, Height);
end;

function TcxSideBarViewInfo.GetCanvas: TcxCanvas;
begin
  Result := Control.Canvas;
end;

function TcxSideBarViewInfo.GetFlat: Boolean;
begin
  Result := Control.LookAndFeel = sblfFlat;
end;

function TcxSideBarViewInfo.GetGroupBorderSize: Integer;
begin
  if Flat then
    Result := 1
  else
    Result := 2;
end;

function TcxSideBarViewInfo.GetGroupViewInfo(Index: Integer): TcxSideBarGroupViewInfo;
begin
  Result := FGroupsViewInfo^[Index];
end;

function TcxSideBarViewInfo.GetLinksAreaBounds: TRect;
begin
  Result := ActiveGroupContentBounds;
  InflateRect(Result, -LinkOffsetX, -LinkOffsetY);
end;

function TcxSideBarViewInfo.GetLinkViewInfo(Index: Integer): TcxSideBarLinkViewInfo;
begin
  Result := FLinksViewInfo^[Index];
end;

function TcxSideBarViewInfo.GetScrollDownBounds: TRect;
begin
  with Result do
  begin
    Right := ActiveGroupContentBounds.Right - ScrollOffset;
    Left := Right - ScrollSize;
    Bottom := ActiveGroupContentBounds.Bottom - ScrollOffset;
    Top := Bottom - ScrollSize;
  end;
end;

function TcxSideBarViewInfo.GetScrollUpBounds: TRect;
begin
  with Result do
  begin
    Right := ActiveGroupContentBounds.Right - ScrollOffset;
    Left := Right - ScrollSize;
    Top := ActiveGroupContentBounds.Top + ScrollOffset;
    Bottom := Top + ScrollSize;
  end;
end;

procedure TcxSideBarViewInfo.CalcGroupsViewInfo;
var
  GroupHeaderHeight, LinksAreaSize, I: Integer;
  R: TRect;
begin
  Canvas.Font := Control.GroupFont;

  GroupHeaderHeight :=
    2 * (GroupBorderSize + GroupTextOffset) + Canvas.TextHeight('Qq');

  R := Bounds;
  InflateRect(R, -BorderSize, -BorderSize);
  FActiveGroupContentBounds := R;
  LinksAreaSize :=
    (R.Bottom - R.Top) - Control.VisibleGroupCount * GroupHeaderHeight;
  if LinksAreaSize < 0 then LinksAreaSize := 0;

  for I := 0 to Control.VisibleGroupCount - 1 do
  begin
    R.Bottom := R.Top + GroupHeaderHeight;
    FGroupsViewInfo^[I].Bounds := R;
    R.Top := R.Bottom;
    if I = Control.ActiveGroup.VisibleIndex then
    begin
      FActiveGroupContentBounds.Top := R.Top;
      Inc(R.Top, LinksAreaSize);
      FActiveGroupContentBounds.Bottom := R.Top;
    end;
  end;
end;

procedure TcxSideBarViewInfo.CalcLinksViewInfo;
var
  LinkCount, I: Integer;
  GroupStyle: TcxSideBarStyle;
  ImageSize: TPoint;
  R: TRect;
  Link: TcxSideBarLink;

  procedure Init;
  begin
    GroupStyle := Control.ActiveGroup.Style;
    ImageSize := GetImageSize(Control.ActiveGroup);
    R := LinksAreaBounds;
    FVisibleLinkCount := LinkCount - Control.TopLinkIndex;
    Canvas.Font := Control.Font;
  end;

  procedure CalcImageAndTextBounds(LinkViewInfo: TcxSideBarLinkViewInfo;
    const InitiativeBounds: TRect);
  const
    Alignments: array[TcxSideBarStyle] of Integer = (cxAlignHCenter, cxAlignLeft);

    procedure InitBounds;
    begin
      with LinkViewInfo do
      begin
        CaptionBounds := InitiativeBounds;
        ImageBounds := InitiativeBounds;
        if GroupStyle = ssLargeImages then
        begin
          with ImageBounds do
          begin
            Left := (Left + Right - ImageSize.X) div 2;
            Right := Left + ImageSize.X;
            Bottom := Top + ImageSize.Y;
          end;
          CaptionBounds.Top := ImageBounds.Bottom + TextOffsetForLargeImages;
        end
        else
        begin
          with ImageBounds do
          begin
            Right := Left + ImageSize.X;
            Bottom := Top + ImageSize.Y;
          end;
          CaptionBounds.Left := ImageBounds.Right + TextOffsetForSmallImages;
        end;
        CaptionBounds.Bottom := CaptionBounds.Top;
      end;  
    end;

    procedure FinishBounds;
    begin
      with LinkViewInfo do
        if GroupStyle = ssLargeImages then
          CaptionBoundsForMouse.Top := ImageBounds.Bottom
        else
          with ImageBounds do
          begin
            CaptionBoundsForMouse.Left := Right;
            if CaptionBounds.Bottom - CaptionBounds.Top > Bottom - Top then
              OffsetRect(ImageBounds, 0,
                (CaptionBounds.Bottom - CaptionBounds.Top - (Bottom - Top)) div 2)
            else
              OffsetRect(CaptionBounds, 0,
                (Bottom - Top - (CaptionBounds.Bottom - CaptionBounds.Top)) div 2);
          end;
    end;

  begin
    InitBounds;
    Canvas.TextExtent(Link.Caption, LinkViewInfo.CaptionBounds,
      Alignments[GroupStyle] or cxAlignTop or cxWordBreak);
    if not Link.Enabled then
      with LinkViewInfo.CaptionBounds do
      begin
        Inc(Right);
        Inc(Bottom);
      end;
    with LinkViewInfo do
      CaptionBoundsForMouse := CaptionBounds;
    FinishBounds;
  end;

  function GetMostBottomBound(LinkViewInfo: TcxSideBarLinkViewInfo): Integer;
  begin
    with LinkViewInfo do
      if CaptionBounds.Bottom > ImageBounds.Bottom then
        Result := CaptionBounds.Bottom - Byte(not Link.Enabled)
      else
        Result := ImageBounds.Bottom;
  end;

begin
  FCanScrollDown := False;
  FCanScrollUp := False;
  if Control.ActiveGroup = nil then
    LinkCount := 0
  else
    LinkCount := Control.ActiveGroup.Links.VisibleCount;
  if LinkCount <> 0 then
  begin
    Init;
    FCanScrollUp := Control.TopLinkIndex <> 0;
    for I := Control.TopLinkIndex to LinkCount - 1 do
    begin
      Link := Control.ActiveGroup.Links.VisibleItems[I];
      CalcImageAndTextBounds(LinkViewInfos[I], R);
      R.Top := GetMostBottomBound(LinkViewInfos[I]) + LinkOffsetY;
      if R.Top > ActiveGroupContentBounds.Bottom - Byte(I <> LinkCount - 1) then
      begin
        FCanScrollDown := True;
        FVisibleLinkCount := I - Control.TopLinkIndex + 1;
        Break;
      end;
    end;
  end;
end;

procedure TcxSideBarViewInfo.CreateViewInfo;

  procedure CreateGroupsViewInfo;
  var
    I: Integer;
  begin
    FStoredGroupCount := Control.VisibleGroupCount;
    if FStoredGroupCount <> 0 then
    begin
      GetMem(FGroupsViewInfo, FStoredGroupCount * SizeOf(TcxSideBarGroupViewInfo));
      for I := 0 to FStoredGroupCount - 1 do
        FGroupsViewInfo^[I] := TcxSideBarGroupViewInfo.Create(Self);
    end;
  end;

  procedure CreateLinksViewInfo;
  var
    I: Integer;
  begin
    if Control.ActiveGroup <> nil then
    begin
      FStoredLinkCount := Control.ActiveGroup.Links.VisibleCount;
      if FStoredLinkCount <> 0 then
      begin
        GetMem(FLinksViewInfo, FStoredLinkCount * SizeOf(TcxSideBarLinkViewInfo));
        for I := 0 to FStoredLinkCount - 1 do
          FLinksViewInfo^[I] := TcxSideBarLinkViewInfo.Create(Self);
      end;
    end;
  end;

begin
  DestroyViewInfo;
  CreateGroupsViewInfo;
  CreateLinksViewInfo;
  FVisibleLinkCount := 0;
end;

procedure TcxSideBarViewInfo.DestroyViewInfo;

  procedure DestroyGroupsViewInfo;
  var
    I: Integer;
  begin
    if FGroupsViewInfo <> nil then
    begin
      for I := 0 to FStoredGroupCount - 1 do
        FGroupsViewInfo^[I].Free;
      FreeMem(FGroupsViewInfo);
      FGroupsViewInfo := nil;
    end;
  end;

  procedure DestroyLinksViewInfo;
  var
    I: Integer;
  begin
    if FLinksViewInfo <> nil then
    begin
      for I := 0 to FStoredLinkCount - 1 do
        FLinksViewInfo^[I].Free;
      FreeMem(FLinksViewInfo);
      FLinksViewInfo := nil;
    end;
  end;

begin
  DestroyLinksViewInfo;
  DestroyGroupsViewInfo;
end;

function TcxSideBarViewInfo.GetImageSize(AGroup: TcxSideBarGroup): TPoint;

  function GetLargeImageSize: TPoint;
  begin
    with Control do
      if LargeImages = nil then
        Result := Point(DefaultLargeImageSize, DefaultLargeImageSize)
      else
        Result := Point(LargeImages.Width, LargeImages.Height);
  end;

  function GetSmallImageSize: TPoint;
  begin
    with Control do
      if SmallImages = nil then
        Result := Point(DefaultSmallImageSize, DefaultSmallImageSize)
      else
        Result := Point(SmallImages.Width, SmallImages.Height);
  end;

begin
  if AGroup.Style = ssLargeImages then
    Result := GetLargeImageSize
  else
    Result := GetSmallImageSize;
  Inc(Result.X, 2 * ImageOffset);
  Inc(Result.Y, 2 * ImageOffset);
end;

procedure TcxSideBarViewInfo.CalcViewInfo;
begin
  CreateViewInfo;
  CalcGroupsViewInfo;
  CalcLinksViewInfo;
end;

function TcxSideBarViewInfo.GetHitTestInfo(X, Y: Integer): TcxSideBarHitTestInfo;
var
  P: TPoint;

  procedure ProcessLinks;
  var
    I: Integer;
    OverImage: Boolean;
  begin
    I := Control.TopLinkIndex;
    for I := I to I + VisibleLinkCount - 1 do
      if LinkViewInfos[I].WantsMouse(P, OverImage) then
      begin
        Result.Link := Control.ActiveGroup.Links.VisibleItems[I];
        if OverImage then
          Result.Code := htLinkImage
        else
          Result.Code := htLinkCaption;
        Break;
      end;
  end;

  procedure ProcessLinkInsertionPositions;
  var
    I, MaxI: Integer;
    R: TRect;
  begin
    if Control.ActiveGroup = nil then
      MaxI := 0
    else
      MaxI := Control.ActiveGroup.Links.VisibleCount;
    I := Control.TopLinkIndex;
    for I := I to I + VisibleLinkCount do
    begin
      R := GetLinkInsertionPosBounds(I);
      if I = MaxI then
        R.Bottom := LinksAreaBounds.Bottom + LinkOffsetY;
      if PtInRect(R, P) then
      begin
        if I < MaxI then
          Result.Link := Control.ActiveGroup.Links.VisibleItems[I];
        Result.Code := htLinkInsertionPos;
        Break;
      end;
    end;
  end;

  procedure ProcessGroups;
  var
    I: Integer;
  begin
    for I := 0 to Control.VisibleGroupCount - 1 do
      if GroupViewInfos[I].WantsMouse(P) then
      begin
        Result.Group := Control.VisibleGroups[I];
        Result.Code := htGroup;
        Break;
      end;
  end;

begin
  with Result do
  begin
    Code := htNone;
    Group := nil;
    Link := nil;
  end;
  CalcViewInfo;
  P := Point(X, Y);
  if PtInRect(Bounds, P) then
    if PtInRect(ActiveGroupContentBounds, P) then
      if Control.ScrollUp.Visible and PtInRect(ScrollUpBounds, P) then
        Result.Code := htScrollUp
      else
        if Control.ScrollDown.Visible and PtInRect(ScrollDownBounds, P) then
          Result.Code := htScrollDown
        else
        begin
          ProcessLinks;
          if Result.Code <> htNone then Exit;
          //if Control.InternalDragging then
            ProcessLinkInsertionPositions;
          if Result.Code = htNone then
            Result.Code := htBackground;
        end
    else
      ProcessGroups;
end;

function TcxSideBarViewInfo.GetLinkInsertionPosBounds(Pos: Integer): TRect;
var
  I, MaxBottom: Integer;
  Link: TcxSideBarLink;
begin
  Result := LinksAreaBounds;
  Result.Bottom := Result.Top;
  Dec(Result.Top, LinkOffsetY);
  for I := Control.TopLinkIndex to Pos - 1 do
  begin
    Link := Control.ActiveGroup.Links.VisibleItems[I];
    MaxBottom := Max(
      GetRect(nil, Link, htLinkImage).Bottom,
      GetRect(nil, Link, htLinkCaption).Bottom);
    OffsetRect(Result, 0, MaxBottom - Result.Top);
  end;
end;

function TcxSideBarViewInfo.GetRect(Group: TcxSideBarGroup; Link: TcxSideBarLink;
  Part: TcxSideBarHitTestCode): TRect;
begin
  case Part of
    htNone:
      Result := Bounds;
    htBackground:
      Result := ActiveGroupContentBounds;
    htGroup:
      if Group.VisuallyVisible then
        Result := GroupViewInfos[Group.VisibleIndex].Bounds;
    htLinkImage:
      if Link.VisuallyVisible then
        Result := LinkViewInfos[Link.VisibleIndex].ImageBounds
      else
        Result := Rect(0, 0, 0, 0);
    htLinkCaption:
      if Link.VisuallyVisible then
        Result := LinkViewInfos[Link.VisibleIndex].CaptionBounds
      else
        Result := Rect(0, 0, 0, 0);
    htLinkInsertionPos:
      Result :=
        GetLinkInsertionPosBounds(Control.GetLinkInsertionPosForLink(Link));
    htScrollUp:
      Result := ScrollUpBounds;
    htScrollDown:
      Result := ScrollDownBounds;
  end;
end;

function TcxSideBarViewInfo.GetScrollBounds(AScroll: TcxSideBarSCroll): TRect;
begin
  if AScroll.IsScrollUp then
    Result := ScrollUpBounds
  else
    Result := ScrollDownBounds;
end;

{ TcxSideBarPainter }

constructor TcxSideBarPainter.Create(AControl: TcxSideBar);
begin
  inherited Create;
  FControl := AControl;
end;

function TcxSideBarPainter.GetBackground: TcxSideBarBackground;
begin
  Result := Control.Background;
end;

function TcxSideBarPainter.GetCanvas: TcxCanvas;
begin
  Result := Control.Canvas;
end;

function TcxSideBarPainter.GetFlat: Boolean;
begin
  Result := ViewInfo.Flat;
end;

function TcxSideBarPainter.GetViewInfo: TcxSideBarViewInfo;
begin
  Result := Control.ViewInfo;
end;

procedure TcxSideBarPainter.DrawBackground;
var
  R: TRect;
begin
  R := ViewInfo.ActiveGroupContentBounds;
  with Canvas, R do
  begin
    IntersectClipRect(R);
    if Top = Bottom then Exit;
    if Flat then
    begin
      Brush.Color := clBtnText;
      FillRect(Rect(Left, Top, Left + 1, Bottom));
      Inc(Left);
      IntersectClipRect(R);
    end;
  end;
  case Background.Mode of
    bmNone:
      DrawBackgroundNone(R);
    bmTexture:
      DrawBackgroundTexture(R);
    bmGradientHorz:
      DrawGradientHorz(R);
    bmGradientVert:
      DrawGradientVert(R);
  end;
end;

procedure TcxSideBarPainter.DrawBackgroundNone(const R: TRect);
begin
  with Canvas do
  begin
    Brush.Color := Control.Color;
    FillRect(R);
  end;
end;

procedure TcxSideBarPainter.DrawBackgroundTexture(const R: TRect);
var
  RWidth, RHeight, ColCount, RowCount, I, J: Integer;
begin
  if Background.Texture.Empty then
  begin
    DrawBackgroundNone(R);
    Exit;
  end;
  with R, Background.Texture do
  begin
    RWidth := Right - Left;
    RHeight := Bottom - Top;
    ColCount := RWidth div Width + Byte(RWidth mod Width <> 0);
    RowCount := RHeight div Height + Byte(RHeight mod Height <> 0);
  end;
  for I := 0 to ColCount - 1 do
    for J := 0 to RowCount - 1 do
      Canvas.Draw(
        R.Left + I * Background.Texture.Width,
        R.Top + J * Background.Texture.Height,
        Background.Texture);
end;

procedure TcxSideBarPainter.DrawBorder;
var
  R: TRect;
begin
  R := ViewInfo.Bounds;
  Canvas.DrawEdge(R, True, True);
  InflateRect(R, -1, -1);
  if not Flat then
  begin
    Canvas.DrawEdge(R, True, False);
    InflateRect(R, -1, -1);
  end;
  Canvas.IntersectClipRect(R);
end;

procedure TcxSideBarPainter.DrawGradient(R: TRect; Vertical: Boolean;
  FromColor, ToColor: TColor);
var
  FromR, FromG, FromB, ToR, ToG, ToB: Byte;
  Interval, I: Integer;

  procedure CalcRGBColors;
  begin
    FromColor := ColorToRGB(FromColor);
    FromR := GetRValue(FromColor);
    FromG := GetGValue(FromColor);
    FromB := GetBValue(FromColor);
    ToColor := ColorToRGB(ToColor);
    ToR := GetRValue(ToColor);
    ToG := GetGValue(ToColor);
    ToB := GetBValue(ToColor);
  end;

  procedure PrepareDrawing;
  begin
    if Vertical then
    begin
      Interval := R.Bottom - R.Top;
      R.Bottom := R.Top + 1;
    end
    else
    begin
      Interval := R.Right - R.Left;
      R.Right := R.Left + 1;
    end;
  end;

  function GetCurColor: TColor;
  begin
    Result :=
      RGB(
        FromR + MulDiv(ToR - FromR, I, Interval - 1),
        FromG + MulDiv(ToG - FromG, I, Interval - 1),
        FromB + MulDiv(ToB - FromB, I, Interval - 1));
  end;

begin
  CalcRGBColors;
  PrepareDrawing;
  for I := 0 to Interval - 1 do
  begin
    with Canvas do
    begin
      Brush.Color := GetCurColor;
      Brush.Style := bsSolid;  ///!!!
      FillRect(R);
    end;
    if Vertical then
      OffsetRect(R, 0, 1)
    else
      OffsetRect(R, 1, 0);
  end;
end;

procedure TcxSideBarPainter.DrawGradientHorz(const R: TRect);
begin
  if Background.MiddleColor = clNone then
    DrawGradient(R, False, Background.BeginColor, Background.EndColor)
  else
    with R do
    begin
      DrawGradient(
        Rect(Left, Top, (Left + Right) div 2, Bottom), False,
        Background.BeginColor, Background.MiddleColor);
      DrawGradient(
        Rect((Left + Right) div 2, Top, Right, Bottom), False,
        Background.MiddleColor, Background.EndColor);
    end;
end;

procedure TcxSideBarPainter.DrawGradientVert(const R: TRect);
begin
  if Background.MiddleColor = clNone then
    DrawGradient(R, True, Background.BeginColor, Background.EndColor)
  else
    with R do
    begin
      DrawGradient(
        Rect(Left, Top, Right, (Top + Bottom) div 2), True,
        Background.BeginColor, Background.MiddleColor);
      DrawGradient(
        Rect(Left, (Top + Bottom) div 2, Right, Bottom), True,
        Background.MiddleColor, Background.EndColor);
    end;
end;

procedure TcxSideBarPainter.DrawGroup(AGroup: TcxSideBarGroup);
var
  R: TRect;
  DecreasedRect: Boolean;
begin
  R := ViewInfo.GetRect(AGroup, nil, htGroup);
  with Canvas, R do
  begin
    DecreasedRect :=
      Flat and AGroup.Active and (AGroup.ViewState = vsNone);
    if DecreasedRect then
      with R do
      begin
        Dec(Bottom);
        Canvas.Brush.Color := clBtnText;
        Canvas.FillRect(Rect(Left, Bottom, Right, Bottom + 1));
      end;
    DrawItemBorder(R, AGroup.ViewState, True);
    Brush.Color := clBtnFace;
    FillRect(R);
    if DecreasedRect then Inc(R.Bottom);
    DrawText(AGroup.Caption, R,
      cxAlignHCenter or cxAlignVCenter or cxSingleLine);
  end;
end;

procedure TcxSideBarPainter.DrawGroups;
var
  I: Integer;
  Group: TcxSideBarGroup;
begin
  Canvas.Font := Control.GroupFont;
  for I := 0 to Control.VisibleGroupCount - 1 do
  begin
    Group := Control.VisibleGroups[I];
    DrawGroup(Group);
    Canvas.ExcludeClipRect(ViewInfo.GetRect(Group, nil, htGroup));
  end;
end;

procedure TcxSideBarPainter.DrawItemBorder(var R: TRect;
  ViewState: TcxSideBarViewState; ReverseOrder: Boolean);

  procedure DrawEdgesAndInflateRect(Sunken, ReverseOrder, Flat: Boolean);
  begin
    if not Flat then
    begin
      Canvas.DrawEdge(R, Sunken, not ReverseOrder);
      InflateRect(R, -1, -1);
    end;
    Canvas.DrawEdge(R, Sunken, ReverseOrder);
    InflateRect(R, -1, -1);
  end;

begin
  case ViewState of
    vsSelected:
      DrawEdgesAndInflateRect(False, False, False);
    vsPressed:
      DrawEdgesAndInflateRect(True, ReverseOrder, False);
  else
    DrawEdgesAndInflateRect(False, False, Flat);
  end;
end;

procedure TcxSideBarPainter.DrawLink(ALink: TcxSideBarLink);
begin
  DrawLinkImage(ALink);
  DrawLinkCaption(ALink);
end;

procedure TcxSideBarPainter.DrawLinkCaption(ALink: TcxSideBarLink);
const
  Alignment: array[Boolean] of Integer = (cxAlignLeft, cxAlignHCenter);
var
  R: TRect;
begin
  R := ViewInfo.GetRect(nil, ALink, htLinkCaption);
  Canvas.Brush.Style := bsClear;
  Canvas.DrawText(ALink.Caption, R,
    Alignment[Control.ActiveGroup.Style = ssLargeImages] or
    cxAlignTop or cxWordBreak, ALink.Enabled);
  Canvas.Brush.Style := bsSolid;
end;

procedure TcxSideBarPainter.DrawLinkImage(ALink: TcxSideBarLink);
var
  R: TRect;
begin
  R := ViewInfo.GetRect(nil, ALink, htLinkImage);
  case ALink.ViewState of
    vsSelected:
      Canvas.DrawEdge(R, False, True);
    vsPressed:
      Canvas.DrawEdge(R, True, False);
  end;
  if ALink.Images <> nil then
    Canvas.DrawImage(ALink.Images, R.Left + ImageOffset, R.Top + ImageOffset,
      ALink.ImageIndex, ALink.Enabled);
end;

procedure TcxSideBarPainter.DrawLinkInsertionPos;
var
  R: TRect;
  P: array[0..3] of TPoint;
begin
  if Control.LinkInsertionPos = -1 then Exit;
  R := ViewInfo.GetRect(nil, Control.LinkInsertionPosLink, htLinkInsertionPos);
  Canvas.Brush.Color := clBtnText;
  Canvas.Pen.Color := clBtnText;
  with R do
  begin
    P[0] := Point(Left, Top);
    P[1] := Point(Left, Bottom - 1);
    P[2] := Point(Left + (Bottom - Top) div 2 - 1, (Top + Bottom) div 2);
    P[3] := Point(P[2].X, P[2].Y - 1);
  end;
  Canvas.Polygon(P);
  with R do
  begin
    P[0].X := Right - 1;
    P[1].X := Right - 1;
    P[2].X := Right - 1 - ((Bottom - Top) div 2 - 1);
    P[3].X := P[2].X;
  end;
  Canvas.Polygon(P);
  with R do
  begin
    Top := P[3].Y;
    Bottom := Top + 2;
  end;
  Canvas.FillRect(R);
end;

procedure TcxSideBarPainter.DrawLinks;
var
  I: Integer;
begin
  if Control.TopLink = nil then Exit;
  Canvas.Font := Control.Font;
  I := Control.TopLinkIndex;
  for I := I to I + ViewInfo.VisibleLinkCount - 1 do
    DrawLink(Control.ActiveGroup.Links.VisibleItems[I]);
end;

procedure TcxSideBarPainter.DrawScroll(AScroll: TcxSideBarScroll);
var
  R: TRect;
begin
  R := ViewInfo.GetScrollBounds(AScroll);
  DrawItemBorder(R, AScroll.ViewState, False);
  Canvas.Brush.Color := clBtnFace;
  Canvas.FillRect(R);
  DrawScrollArrow(R, AScroll.IsScrollUp, AScroll.ViewState = vsPressed);
end;

procedure TcxSideBarPainter.DrawScrollArrow(const R: TRect; IsScrollUp, IsPressed: Boolean);
var
  P: array[0..2] of TPoint;
  ArrowY, AnotherY, I: Integer;
begin
  with R do
  begin
    if IsScrollUp then
    begin
      ArrowY := (Top + Bottom - ScrollArrowHeight) div 2;
      AnotherY := ArrowY + ScrollArrowHeight - 1;
    end
    else
    begin
      ArrowY := (Top + Bottom + ScrollArrowHeight) div 2 - 1;
      AnotherY := ArrowY - (ScrollArrowHeight - 1);
    end;
    P[0] := Point((Left + Right - Byte(not Odd(Right - Left))) div 2, ArrowY);
    P[1] := Point((Left + Right - ScrollArrowWidth) div 2, AnotherY);
    P[2] := Point(P[1].X + ScrollArrowWidth - 1, P[1].Y);
  end;
  if IsPressed then
    for I := 0 to 2 do
      with P[I] do
      begin
        Inc(X);
        Inc(Y);
      end;
  with Canvas do
  begin
    Brush.Color := clBtnText;
    Pen.Color := clBtnText;
    Polygon(P);
  end;
end;

procedure TcxSideBarPainter.DrawScrolls;
begin
  if Control.ScrollUp.Visible then
  begin
    DrawScroll(Control.ScrollUp);
    Canvas.ExcludeClipRect(ViewInfo.ScrollUpBounds);
  end;
  if Control.ScrollDown.Visible then
  begin
    DrawScroll(Control.ScrollDown);
    Canvas.ExcludeClipRect(ViewInfo.ScrollDownBounds);
  end;
end;

procedure TcxSideBarPainter.Invalidate(Group: TcxSideBarGroup;
  Link: TcxSideBarLink; Part: TcxSideBarHitTestCode);
begin
  if Control.FUpdateLockCount = 0 then
    Control.InvalidateRect(ViewInfo.GetRect(Group, Link, Part), False);
end;

procedure TcxSideBarPainter.InvalidateLinkImageBorder(Link: TcxSideBarLink);
begin
  with ViewInfo.GetRect(nil, Link, htLinkImage) do
  begin
    Control.InvalidateRect(Rect(Left, Top, Left + ImageBorderSize, Bottom), False);
    Control.InvalidateRect(Rect(Right - ImageBorderSize, Top, Right, Bottom), False);
    Control.InvalidateRect(Rect(Left, Top, Right, Top + ImageBorderSize), False);
    Control.InvalidateRect(Rect(Left, Bottom - ImageBorderSize, Right, Bottom), False);
  end;
end;

procedure TcxSideBarPainter.Paint;
begin
  DrawBorder;
  DrawGroups;
  DrawScrolls;
  DrawBackground;
  DrawLinkInsertionPos;
  DrawLinks;
end;

{ TcxSideBar }

constructor TcxSideBar.Create(AOwner: TComponent);
begin
  inherited;
  Align := alLeft;
  Color := {$IFDEF LINUX}clGray{$ELSE}clBtnShadow{$ENDIF};
  ControlStyle := ControlStyle + [csOpaque];
  Width := 120;
  FBackground := TcxSideBarBackground.Create(Self);
  FCanvas := TcxCanvas.Create(inherited Canvas);
  FGroupFont := TFont.Create;
  FGroupFont.OnChange := GroupFontChangeHandler;
  FGroups := TList.Create;
  FItems := TList.Create;
  FLargeImagesChangeLink := TChangeLink.Create;
  FLargeImagesChangeLink.OnChange := ImagesChanged;
  FSmallImagesChangeLink := TChangeLink.Create;
  FSmallImagesChangeLink.OnChange := ImagesChanged;
  FLinkInsertionPos := -1;
  FPainter := GetPainterClass.Create(Self);
  FScrollDown := TcxSideBarScrollDown.Create(Self);
  FScrollUp := TcxSideBarScrollUp.Create(Self);
  FScrollTimer := TTimer.Create(nil);
  with FScrollTimer do
  begin
    OnTimer := ScrollTimerHandler;
    Enabled := False;
  end;
  FViewInfo := TcxSideBarViewInfo.Create(Self);
  FVisibleGroups := TList.Create;
  Font.Color := clWindow;
end;

destructor TcxSideBar.Destroy;
begin
  FinishEditing(False);
  InplaceEdit.Free;
  StopScrollTimer;
  FScrollTimer.Free;
  FScrollUp.Free;
  FScrollDown.Free;
  FSmallImagesChangeLink.Free;
  FLargeImagesChangeLink.Free;
  DestroyItems;
  FItems.Free;
  DestroyGroups;
  FVisibleGroups.Free;
  FGroups.Free;
  FGroupFont.Free;
  FViewInfo.Free;
  FPainter.Free;
  FCanvas.Free;
  FBackground.Free;
  inherited;
end;

function TcxSideBar.GetGroup(Index: Integer): TcxSideBarGroup;
begin
  Result := TcxSideBarGroup(FGroups[Index]);
end;

function TcxSideBar.GetGroupCount: Integer;
begin
  Result := FGroups.Count;
end;

function TcxSideBar.GetInternalDragging: Boolean;
begin
  Result := DraggingLink <> nil;
end;

function TcxSideBar.GetItem(Index: Integer): TcxSideBarItem;
begin
  Result := TcxSideBarItem(FItems[Index]);
end;

function TcxSideBar.GetItemCount: Integer;
begin
  Result := FItems.Count;
end;

function TcxSideBar.GetLinkInsertionPosLink: TcxSideBarLink;
begin
  Result := GetLinkForLinkInsertionPos(LinkInsertionPos);
end;

function TcxSideBar.GetStayPressed: Boolean;
begin
  Result := soStayPressed in FOptions;
end;

function TcxSideBar.GetTopLinkIndex: Integer;
begin
  if TopLink = nil then
    Result := 0
  else
    Result := TopLink.VisibleIndex;
end;

function TcxSideBar.GetVisibleGroup(Index: Integer): TcxSideBarGroup;
begin
  Result := TcxSideBarGroup(FVisibleGroups[Index]);
end;

function TcxSideBar.GetVisibleGroupCount: Integer;
begin
  Result := FVisibleGroups.Count;
end;

procedure TcxSideBar.SetActiveGroup(Value: TcxSideBarGroup);
begin
  if (Value = nil) and (VisibleGroupCount <> 0) or
    (Value <> nil) and not Value.Visible then
    Exit;
  if FActiveGroup <> Value then
  begin
    FActiveGroup := Value;
    ActiveGroupChanged;
  end;
end;

procedure TcxSideBar.SetActiveItem(Value: TcxSideBarItem);
var
  PrevActiveItem: TcxSideBarItem;
begin
  if not StayPressed or (FActiveItem <> Value) then
  begin
    FinishEditing(False);
    PrevActiveItem := FActiveItem;
    FActiveItem := Value;
    ActiveItemChanged(PrevActiveItem);
  end;
end;

procedure TcxSideBar.SetAssignedValues(Value: TcxSideBarAssignedValues);
var
  ChangedValues: TcxSideBarAssignedValues;
begin
  if FAssignedValuesChanging then Exit;
  FAssignedValuesChanging := True;
  try
    if FAssignedValues <> Value then
    begin
      Byte(ChangedValues) := Byte(FAssignedValues) xor Byte(Value);
      FAssignedValues := Value;
      if avGroupFont in ChangedValues then
      begin
        if not IsGroupFontAssigned then
          SyncGroupFont;
        GroupFontChanged;
      end;
    end;
  finally
    FAssignedValuesChanging := False;
  end;
end;

procedure TcxSideBar.SetBackground(Value: TcxSideBarBackground);
begin
  FBackground.Assign(Value);
end;

procedure TcxSideBar.SetGroupFont(Value: TFont);
begin
  FGroupFont.Assign(Value);
end;

procedure TcxSideBar.SetLargeImages(Value: TImageList);
begin
  if FLargeImages <> Value then
  begin
    if FLargeImages <> nil then
      FLargeImages.UnregisterChanges(FLargeImagesChangeLink);
    FLargeImages := Value;
    if FLargeImages <> nil then
    begin
      FLargeImages.RegisterChanges(FLargeImagesChangeLink);
      FLargeImages.FreeNotification(Self);
    end;
    Changed(True);
  end;
end;

procedure TcxSideBar.SetLinkInsertionPos(Value: Integer);
var
  PrevLinkInsertionPos: Integer;
  PrevLinkInsertionPosLink: TcxSideBarLink;
begin
  if FLinkInsertionPos <> Value then
  begin
    PrevLinkInsertionPos := LinkInsertionPos;
    PrevLinkInsertionPosLink := LinkInsertionPosLink;
    FLinkInsertionPos := Value;
    if PrevLinkInsertionPos <> -1 then
      Painter.Invalidate(nil, PrevLinkInsertionPosLink, htLinkInsertionPos);
    if FLinkInsertionPos <> -1 then
      Painter.Invalidate(nil, LinkInsertionPosLink, htLinkInsertionPos);
  end;
end;

procedure TcxSideBar.SetLookAndFeel(Value: TcxSideBarLookAndFeel);
begin
  if FLookAndFeel <> Value then
  begin
    FLookAndFeel := Value;
    Changed(True);
  end;
end;

procedure TcxSideBar.SetOptions(Value: TcxSideBarOptions);
begin
  if FOptions <> Value then
  begin
    FOptions := Value;
    Changed(True);
  end;
end;

procedure TcxSideBar.SetPressedGroup(Value: TcxSideBarGroup);
var
  PrevPressedGroup: TcxSideBarGroup;
begin
  if FPressedGroup <> Value then
  begin
    PrevPressedGroup := FPressedGroup;
    FPressedGroup := Value;
    if PrevPressedGroup <> nil then
      PrevPressedGroup.PressedChanged;
    if FPressedGroup <> nil then
      FPressedGroup.PressedChanged;
  end;
end;

procedure TcxSideBar.SetPressedLink(Value: TcxSideBarLink);
var
  PrevPressedLink: TcxSideBarLink;
begin
  if (Value <> nil) and not Value.Enabled then
    Value := nil;
  if FPressedLink <> Value then
  begin
    PrevPressedLink := FPressedLink;
    FPressedLink := Value;
    if PrevPressedLink <> nil then
      PrevPressedLink.PressedChanged;
    if FPressedLink <> nil then
      FPressedLink.PressedChanged;
  end;
end;

procedure TcxSideBar.SetSelectedGroup(Value: TcxSideBarGroup);
var
  PrevSelectedGroup: TcxSideBarGroup;
begin
  if FSelectedGroup <> Value then
  begin
    PrevSelectedGroup := FSelectedGroup;
    FSelectedGroup := Value;
    PressedGroup := nil;
    if PrevSelectedGroup <> nil then
      PrevSelectedGroup.SelectedChanged;
    if FSelectedGroup <> nil then
    begin
      if PrevSelectedGroup = nil then
        SetCursor(crHandPoint);
      FSelectedGroup.SelectedChanged;
    end
    else
      RestoreCursor;
  end;
end;

procedure TcxSideBar.SetSelectedLink(Value: TcxSideBarLink);
var
  PrevSelectedLink: TcxSideBarLink;
begin
  if (Value <> nil) and not Value.Enabled then
    Value := nil;
  if FSelectedLink <> Value then
  begin
    PrevSelectedLink := FSelectedLink;
    FSelectedLink := Value;
    PressedLink := nil;
    if PrevSelectedLink <> nil then
      PrevSelectedLink.SelectedChanged;
    if FSelectedLink <> nil then
      FSelectedLink.SelectedChanged;
  end;
end;

procedure TcxSideBar.SetSmallImages(Value: TImageList);
begin
  if FSmallImages <> Value then
  begin
    if FSmallImages <> nil then
      FSmallImages.UnregisterChanges(FSmallImagesChangeLink);
    FSmallImages := Value;
    if FSmallImages <> nil then
    begin
      FSmallImages.RegisterChanges(FSmallImagesChangeLink);
      FSmallImages.FreeNotification(Self);
    end;
    Changed(True);
  end;
end;

procedure TcxSideBar.SetTopLink(Value: TcxSideBarLink);
begin
  if FTopLink <> Value then
  begin
    FTopLink := Value;
    if ActiveGroup <> nil then ActiveGroup.Changed(True);
  end;
end;

procedure TcxSideBar.SetTopLinkIndex(Value: Integer);
begin
  if (ActiveGroup <> nil) and
    (0 <= Value) and (Value < ActiveGroup.Links.VisibleCount) then
    TopLink := ActiveGroup.Links.VisibleItems[Value];
end;

procedure TcxSideBar.SetStyle(Value: TcxSideBarStyle);
begin
  if FStyle <> Value then
  begin
    FStyle := Value;
    Changed(True);
  end;
end;

procedure TcxSideBar.SetWaitingGroup(Value: TcxSideBarGroup);
begin
  if Value = ActiveGroup then Value := nil;
  if FWaitingGroup <> Value then
  begin
    FWaitingGroup := Value;
    if Value = nil then
    begin
      FWaitingTimer.Free;
      FWaitingTimer := nil;
    end
    else
    begin
      if FWaitingTimer = nil then
      begin
        FWaitingTimer := TTimer.Create(nil);
        FWaitingTimer.Interval := WaitingInterval;
        FWaitingTimer.OnTimer := WaitingTimerHandler;
      end;
      FWaitingTimer.Enabled := False;
      FWaitingTimer.Enabled := True;
    end;
  end;
end;

procedure TcxSideBar.SyncGroupFont;
begin
  GroupFont := Font;
  GroupFont.Color := clBtnText;
end;

procedure TcxSideBar.DestroyGroups;
begin
  while GroupCount <> 0 do
    Groups[GroupCount - 1].Free;
end;

procedure TcxSideBar.DestroyItems;
begin
  while ItemCount <> 0 do
    Items[ItemCount - 1].Free;
end;

procedure TcxSideBar.GroupFontChangeHandler(Sender: TObject);
begin
  if IsGroupFontAssigned then
    GroupFontChanged
  else
    AssignedValues := AssignedValues + [avGroupFont];
end;

procedure TcxSideBar.ImagesChanged(Sender: TObject);
begin
  Changed(True);
end;

function TcxSideBar.IsGroupFontAssigned: Boolean;
begin
  Result := avGroupFont in AssignedValues;
end;

procedure TcxSideBar.MoveGroup(FromIndex, ToIndex: Integer);
begin
  FGroups.Move(FromIndex, ToIndex);
end;

procedure TcxSideBar.MoveItem(FromIndex, ToIndex: Integer);
begin
  FItems.Move(FromIndex, ToIndex);
end;

procedure TcxSideBar.ScrollTimerHandler(Sender: TObject);
begin
  with GetScrollByScrollTag(TTimer(Sender).Tag) do
    if Visible then
      DoClick
    else
      StopScrollTimer;
end;

procedure TcxSideBar.WaitingTimerHandler(Sender: TObject);
begin
  ActiveGroup := WaitingGroup;
  WaitingGroup := nil;
end;

function TcxSideBar.CanDrag(X, Y: Integer): Boolean;
var
  HitTestInfo: TcxSideBarHitTestInfo;
begin
  HitTestInfo := ViewInfo.GetHitTestInfo(X, Y);
  Result := HitTestInfo.Code in LinkHitTestCodes;
end;

procedure TcxSideBar.DoEndDrag(Target: TObject; X, Y: Integer);
begin
  DraggingLink := nil;
  inherited;
end;

procedure TcxSideBar.DragOver(Source: TObject; X, Y: Integer; State: TDragState;
  var Accept: Boolean);
const
  AcceptHitTestCodes = [htGroup, htLinkInsertionPos, htScrollUp, htScrollDown];
var
  HitTestInfo: TcxSideBarHitTestInfo;
  ALinkInsertionPos, ScrollTag: Integer;
  AWaitingGroup: TcxSideBarGroup;

  procedure ProcessScrolls;
  begin
    if ScrollTag = 0 then
    begin
      StopScrollTimer;
      ScrollUp.ViewState := vsNone;
      ScrollDown.ViewState := vsNone;
    end
    else
    begin
      GetScrollByScrollTag(ScrollTag).ViewState := vsPressed;
      StartScrollTimer(ScrollTag, True);
    end;
  end;

begin
  inherited;
  Accept := Accept or InternalDragging;
  HitTestInfo := ViewInfo.GetHitTestInfo(X, Y);
  if Accept and (HitTestInfo.Code in LinkHitTestCodes) then
    SelectedLink := HitTestInfo.Link
  else
    SelectedLink := nil;

  Accept := Accept and (HitTestInfo.Code in AcceptHitTestCodes);
  ScrollTag := 0;
  ALinkInsertionPos := -1;
  AWaitingGroup := nil;

  if Accept and (State <> dsDragLeave) then
    case HitTestInfo.Code of
      htGroup:
        AWaitingGroup := HitTestInfo.Group;
      htLinkInsertionPos:
        ALinkInsertionPos := GetLinkInsertionPosForLink(HitTestInfo.Link);
      htScrollUp:
        ScrollTag := ScrollUpTag;
      htScrollDown:
        ScrollTag := ScrollDownTag;
    end;

  ProcessScrolls;
  LinkInsertionPos := ALinkInsertionPos;
  WaitingGroup := AWaitingGroup;
end;

procedure TcxSideBar.FontChanged;
begin
  inherited;
  if not IsGroupFontAssigned then
  begin
    SyncGroupFont;
    AssignedValues := AssignedValues - [avGroupFont];
  end;
end;

procedure TcxSideBar.GetChildren(Proc: TGetChildProc; Root: TComponent);
var
  I: Integer;
begin
  if Root = Owner then
  begin
    for I := 0 to GroupCount - 1 do
      Proc(Groups[I]);
    for I := 0 to ItemCount - 1 do
      Proc(Items[I]);
  end;
end;

function TcxSideBar.GetDesignHitTest(X, Y: Integer; Shift: TShiftState): Boolean;
const
  AcceptableHitTestCodes =
    [htGroup, htLinkImage, htLinkCaption, htScrollUp, htScrollDown];
begin
  Result := ViewInfo.GetHitTestInfo(X, Y).Code in AcceptableHitTestCodes;
  if not Result then MouseLeave(Self);
end;

function TcxSideBar.MayFocus: Boolean;
begin
  Result := False;
end;

procedure TcxSideBar.MouseDown(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  HitTestInfo: TcxSideBarHitTestInfo;

  procedure ProcessForDesignTime;
  begin
    if ssDouble in Shift then
      cxSideBarDesignerLink.ShowDefaultEventHandler(HitTestInfo.Link.Item)
    else
      cxSideBarDesignerLink.SetSelection(Self, HitTestInfo.Link.Item);
  end;

begin
  FinishEditing(False);
  if Button = mbLeft then
  begin
    HitTestInfo := ViewInfo.GetHitTestInfo(X, Y);
    if HitTestInfo.Code in LinkHitTestCodes then
    begin
      PressedLink := HitTestInfo.Link;
      LastPressedLink := HitTestInfo.Link;
      ProcessForDesignTime;
    end
    else
    begin
      LastPressedLink := nil;
      case HitTestInfo.Code of
        htGroup:
          PressedGroup := HitTestInfo.Group;
        htScrollUp:
          begin
            ScrollUp.ViewState := vsPressed;
            ScrollUp.DoClick;
            StartScrollTimer(ScrollUpTag, False);
          end;
        htScrollDown:
          begin
            ScrollDown.ViewState := vsPressed;
            ScrollDown.DoClick;
            StartScrollTimer(ScrollDownTag, False);
          end;
      end;
    end;
  end;
  inherited;
end;

procedure TcxSideBar.MouseLeave(AControl: TControl);
begin
  inherited;
  SelectedGroup := nil;
  SelectedLink := nil;
  ScrollUp.ViewState := vsNone;
  ScrollDown.ViewState := vsNone;
end;

procedure TcxSideBar.MouseMove(Shift: TShiftState; X, Y: Integer);
const
  ViewStates: array[Boolean, Boolean] of TcxSideBarViewState =
    ((vsNone, vsNone), (vsSelected, vsPressed));
var
  HitTestInfo: TcxSideBarHitTestInfo;
begin
  inherited;
  HitTestInfo := ViewInfo.GetHitTestInfo(X, Y);
  if HitTestInfo.Code = htGroup then
    SelectedGroup := HitTestInfo.Group
  else
    SelectedGroup := nil;
  if HitTestInfo.Code in LinkHitTestCodes then
    SelectedLink := HitTestInfo.Link
  else
    SelectedLink := nil;
  ScrollUp.ViewState :=
    ViewStates[HitTestInfo.Code = htScrollUp, ssLeft in Shift];
  ScrollDown.ViewState :=
    ViewStates[HitTestInfo.Code = htScrollDown, ssLeft in Shift];
end;

procedure TcxSideBar.MouseUp(Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
var
  Group: TcxSideBarGroup;
  Item: TcxSideBarItem;
begin
  inherited;
  if PressedGroup <> nil then
  begin
    Group := PressedGroup;
    PressedGroup := nil;
    if AcceptMousePosForClick(X, Y) then ActiveGroup := Group;
  end;
  if PressedLink <> nil then
  begin
    Item := PressedLink.Item;
    PressedLink := nil;
    if AcceptMousePosForClick(X, Y) then ActiveItem := Item;
  end;
  ScrollUp.ViewState := vsNone;
  ScrollDown.ViewState := vsNone;
end;

procedure TcxSideBar.Notification(AComponent: TComponent; Operation: TOperation); 
begin
  inherited;
  if Operation = opRemove then
  begin
    if AComponent = LargeImages then LargeImages := nil;
    if AComponent = SmallImages then SmallImages := nil;
  end;
end;

procedure TcxSideBar.Paint;
begin
  if FUpdateLockCount <> 0 then Exit;
  ViewInfo.CalcViewInfo;
  CheckTopLink;
  Painter.Paint;
end;

procedure TcxSideBar.Resize;
begin
  inherited;
  FinishEditing(False);
end;

procedure TcxSideBar.SetName(const NewName: TComponentName);
var
  OldName: TComponentName;

  procedure RenameComponents(IsItems: Boolean);
  var
    I: Integer;
    Component: TComponent;
    ComponentName, NamePrefix: TComponentName;

    function GetComponentCount: Integer;
    begin
      if IsItems then
        Result := ItemCount
      else
        Result := GroupCount;
    end;

    function GetComponent(Index: Integer): TComponent;
    begin
      if IsItems then
        Result := Items[Index]
      else
        Result := Groups[Index];
    end;

  begin
    for I := 0 to GetComponentCount - 1 do
    begin
      Component := GetComponent(I);
      if Component.Owner = Owner then
      begin
        ComponentName := Component.Name;
        if Length(ComponentName) > Length(OldName) then
        begin
          NamePrefix := Copy(ComponentName, 1, Length(OldName));
          if SameText(OldName, NamePrefix) then
          begin
            Delete(ComponentName, 1, Length(OldName));
            Insert(NewName, ComponentName, 1);
            try
              Component.Name := ComponentName;
            except
              on EComponentError do { Ignore rename errors };
            end;
          end;
        end;
      end;
    end;
  end;

begin
  OldName := Name;
  inherited;
  if IsDesigning and (Name <> OldName) then
  begin
    RenameComponents(True);
    RenameComponents(False);
  end;
  cxSideBarDesignerLink.ControlNameChanged(Self);
end;

function TcxSideBar.StartDrag(var DragObject: TDragObject): Boolean;
begin
  Result := inherited StartDrag(DragObject);
//  Result := LastPressedLink <> nil;
  if Result then
    DraggingLink := LastPressedLink;
end;

procedure TcxSideBar.ActiveGroupChanged;
begin
  FinishEditing(False);
  TopLink := FindTopLink;
  Changed(True);
  cxSideBarDesignerLink.Modified(Self);
end;

procedure TcxSideBar.ActiveItemChanged(PrevActiveItem: TcxSideBarItem);
begin
  if (PrevActiveItem <> nil) and (PrevActiveItem <> FActiveItem) then
    PrevActiveItem.ActiveChanged;
  if FActiveItem <> nil then
    FActiveItem.ActiveChanged;
  cxSideBarDesignerLink.Modified(Self);  
end;

procedure TcxSideBar.Changed(ViewInfoChanged: Boolean);
begin
  Painter.Invalidate(nil, nil, htNone);
end;

procedure TcxSideBar.CheckTopLink;
begin
  BeginUpdate;
  try
    while (TopLinkIndex <> 0) and not ViewInfo.CanScrollDown do
    begin
      TopLinkIndex := TopLinkIndex - 1;
      ViewInfo.CalcViewInfo;
      if ViewInfo.CanScrollDown then
      begin
        TopLinkIndex := TopLinkIndex + 1;
        ViewInfo.CalcViewInfo;
        Break;
      end;
    end;
  finally
    CancelUpdate;
  end;
end;

procedure TcxSideBar.DoGroupCreated(AGroup: TcxSideBarGroup);
begin
  FGroups.Add(AGroup);
  AGroup.VisibleChanged;
end;

procedure TcxSideBar.DoGroupDeleted(AGroup: TcxSideBarGroup);
begin
  FGroups.Remove(AGroup);
  AGroup.VisibleChanged;
end;

procedure TcxSideBar.DoItemCreated(AItem: TcxSideBarItem);
begin
  FItems.Add(AItem);
end;

procedure TcxSideBar.DoItemDeleted(AItem: TcxSideBarItem);
begin
  FItems.Remove(AItem);
  if ActiveItem = AItem then ActiveItem := nil;
  AItem.VisibleChanged;
end;

function TcxSideBar.FindActiveGroup: TcxSideBarGroup;
begin
  if VisibleGroupCount = 0 then
    Result := nil
  else
    Result := VisibleGroups[0];
end;

function TcxSideBar.FindTopLink: TcxSideBarLink;
begin
  if (ActiveGroup = nil) or (ActiveGroup.Links.VisibleCount = 0) then
    Result := nil
  else
    Result := ActiveGroup.Links.VisibleItems[0];
end;

function TcxSideBar.GetLinkForLinkInsertionPos(Pos: Integer): TcxSideBarLink;
begin
  if (ActiveGroup <> nil) and
    (0 <= Pos) and (Pos < ActiveGroup.Links.VisibleCount) then
    Result := ActiveGroup.Links.VisibleItems[Pos]
  else
    Result := nil;
end;

function TcxSideBar.GetLinkInsertionPosForLink(Link: TcxSideBarLink): Integer;
begin
  if Link = nil then
    if ActiveGroup = nil then
      Result := 0
    else
      Result := ActiveGroup.Links.VisibleCount
  else
    Result := Link.VisibleIndex;
end;

function TcxSideBar.GetPainterClass: TcxSideBarPainterClass;
begin
  Result := TcxSideBarPainter;
end;

procedure TcxSideBar.GroupFontChanged;
begin
  Changed(True);
end;

procedure TcxSideBar.RefreshVisibleGroups;
var
  I: Integer;
begin
  FVisibleGroups.Clear;
  for I := 0 to GroupCount - 1 do
    if Groups[I].Visible then
      FVisibleGroups.Add(Groups[I]);
end;

function TcxSideBar.GetScrollByScrollTag(ScrollTag: Integer): TcxSideBarScroll;
begin
  case ScrollTag of
    ScrollUpTag:
      Result := ScrollUp;
    ScrollDownTag:
      Result := ScrollDown;
  else
    Result := nil;
  end;
end;

procedure TcxSideBar.StartScrollTimer(ScrollTag: Integer; IsDragDrop: Boolean);
begin
  with FScrollTimer do
    if Tag <> ScrollTag then
    begin
      Tag := ScrollTag;
      Enabled := True;
      if IsDragDrop then
        Interval := ScrollTimeDragDropInterval
      else
        Interval := ScrollTimeInterval;
    end;
end;

procedure TcxSideBar.StopScrollTimer;
begin
  with FScrollTimer do
    if Tag <> 0 then
    begin
      Tag := 0;
      Enabled := False;
    end;
end;

procedure TcxSideBar.SetCursor(ACursor: TCursor);
begin
  FSaveCursor := Cursor;
  Cursor := ACursor;
end;

procedure TcxSideBar.RestoreCursor;
begin
  Cursor := FSaveCursor;
end;

procedure TcxSideBar.DragDrop(Source: TObject; X, Y: Integer);
var
  HitTestInfo: TcxSideBarHitTestInfo;
  ALinkInsertionPos: Integer;
  Link: TcxSideBarLink;
begin
  inherited;
  HitTestInfo := ViewInfo.GetHitTestInfo(X, Y);
  if HitTestInfo.Code = htLinkInsertionPos then
    ALinkInsertionPos := GetLinkInsertionPosForLink(HitTestInfo.Link)
  else
    ALinkInsertionPos := -1;
  if (ALinkInsertionPos <> -1) and InternalDragging then
  begin
    Link := ActiveGroup.Links.Add;
    Link.Assign(DraggingLink);
    Link.VisibleIndex := ALinkInsertionPos;
    if not IsCopyDragDrop then
      DraggingLink.Free;
    Link.MakeVisuallyVisible;
  end;
end;

function TcxSideBar.AddGroup: TcxSideBarGroup;
begin
  Result := TcxSideBarGroup.Create(Owner);
  Result.Control := Self;
end;

function TcxSideBar.AddItem: TcxSideBarItem;
begin
  Result := TcxSideBarItem.Create(Owner);
  Result.Control := Self;
end;

procedure TcxSideBar.BeginUpdate;
begin
  Inc(FUpdateLockCount);
end;

procedure TcxSideBar.CancelUpdate;
begin
  if FUpdateLockCount <> 0 then
    Dec(FUpdateLockCount);
end;

procedure TcxSideBar.EndUpdate;
begin
  if FUpdateLockCount <> 0 then
  begin
    Dec(FUpdateLockCount);
    if FUpdateLockCount = 0 then Changed(True);
  end;
end;

procedure TcxSideBar.StartEditing(AEditingObject: TObject);
begin
  if InplaceEdit = nil then
    FInplaceEdit := TcxSideBarInplaceEdit.Create(Self);
  InplaceEdit.EditingObject := AEditingObject;
  InplaceEdit.StartEditing;
end;

procedure TcxSideBar.FinishEditing(Accept: Boolean);
begin
  if InplaceEdit <> nil then
    InplaceEdit.FinishEditing(Accept);
end;

initialization
  RegisterClasses([TcxSideBarItem, TcxSideBarGroup]);

  cxSideBarDesignerLink := TcxSideBarDesignerLink.Create;

finalization
  cxSideBarDesignerLink.Free;
  cxSideBarDesignerLink := nil;

end.
