{**********************************************************************}
{                                                                      }
{       Developer Express Visual Component Library                     }
{       ExpressSideBar - CLX/VCL Edition                               }
{                                                                      }
{       Copyright (c) 1998-2001 Developer Express Inc.                 }
{       ALL RIGHTS RESERVED                                            }
{                                                                      }
{   The entire contents of this file is protected by U.S. and          }
{   International Copyright Laws. Unauthorized reproduction,           }
{   reverse-engineering, and distribution of all or any portion of     }
{   the code contained in this file is strictly prohibited and may     }
{   result in severe civil and criminal penalties and will be          }
{   prosecuted to the maximum extent possible under the law.           }
{                                                                      }
{   RESTRICTIONS                                                       }
{                                                                      }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES              }
{   (DCU, OBJ, DLL, DPU, SO, ETC.) ARE CONFIDENTIAL AND PROPRIETARY    }
{   TRADE SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER   }
{   IS LICENSED TO DISTRIBUTE THE EXPRESSSIDEBAR                       }
{   AS PART OF AN EXECUTABLE PROGRAM ONLY.                             }
{                                                                      }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED         }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE           }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE          }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT     }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                         }
{                                                                      }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON          }
{   ADDITIONAL RESTRICTIONS.                                           }
{                                                                      }
{**********************************************************************}

unit cxSideBarDesigner;

{$I cxSideBarVer.inc}

interface

uses
  Classes,
{$IFDEF VCL}
  Windows, {$IFDEF DELPHI6}DesignIntf, DesignWindows, {$ELSE}DsgnIntf, DsgnWnds, {$ENDIF}
  Graphics, Controls, Forms, StdCtrls,
{$ELSE}
  DesignIntf, ClxDesignWindows, Types, QGraphics, QControls, QForms, QStdCtrls,
{$ENDIF}
  cxSideBar;

type
  TDesigner = {$IFDEF DELPHI6}IDesigner{$ELSE}IFormDesigner{$ENDIF};
  TDesignerSelection = {$IFDEF DELPHI6}IDesignerSelections{$ELSE}TDesignerSelectionList{$ENDIF};

  TcxSideBarDesigner = class;

  TcxSideBarDesigners = class
  private
    FItems: TList;
    FLastPos: TPoint;
    function GetCount: Integer;
    function GetDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
    function GetIsCustomizing(AControl: TcxSideBar): Boolean;
    function GetItem(Index: Integer): TcxSideBarDesigner;
  protected
    function CreateDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
    function FindDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
    procedure DoDesignerCreated(ADesigner: TcxSideBarDesigner);
    procedure DoDesignerDeleted(ADesigner: TcxSideBarDesigner);

    property Count: Integer read GetCount;
    property Items[Index: Integer]: TcxSideBarDesigner read GetItem;
    property LastPos: TPoint read FLastPos write FLastPos;
  public
    constructor Create;
    destructor Destroy; override;

    procedure ControlNameChanged(AControl: TcxSideBar);
    procedure GroupNameChanged(AControl: TcxSideBar);
    procedure ItemNameChanged(AControl: TcxSideBar);
    procedure LinkItemChanged(AControl: TcxSideBar);

    procedure ShowDesigner(AControl: TcxSideBar; ADesigner: TDesigner);
    property Designers[AControl: TcxSideBar]: TcxSideBarDesigner read GetDesigner; default;
    property IsCustomizing[AControl: TcxSideBar]: Boolean read GetIsCustomizing;
  end;

  TcxSideBarDesignerObjects = class
  private
    FCanAdd: Boolean;
    FCanDelete: Boolean;
    FDesigner: TcxSideBarDesigner;
    function GetControl: TcxSideBar;
    function GetSelectedCount: Integer;
    function GetSelectedItem: TPersistent;
    function GetSelectedItems: TDesignerSelection;
    procedure SetSelectedItem(Value: TPersistent);
    procedure SetSelectedItems(Value: TDesignerSelection);
    procedure ListBoxClick(Sender: TObject);
    procedure ListBoxDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ListBoxDragOver(Sender, Source: TObject; X, Y: Integer;
      State: TDragState; var Accept: Boolean);
    procedure ListBoxKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure ListBoxStartDrag(Sender: TObject; var DragObject: TDragObject);
  protected
    function ChangeItemIndex(AItem: TPersistent; NewIndex: Integer): Integer; virtual; abstract;
    function GetBaseName: string; virtual;
    function GetButtonAdd: TButton; virtual; abstract;
    function GetButtonDelete: TButton; virtual; abstract;
    function GetCount: Integer; virtual; abstract;
    function GetDisplayText(AObject: TPersistent): string; virtual;
    function GetItem(Index: Integer): TPersistent; virtual; abstract;
    function GetListBox: TListBox; virtual; abstract;
    procedure SelectionChanged; virtual;

    property BaseName: string read GetBaseName;
    property ButtonAdd: TButton read GetButtonAdd;
    property ButtonDelete: TButton read GetButtonDelete;
    property CanAdd: Boolean read FCanAdd write FCanAdd;
    property CanDelete: Boolean read FCanDelete write FCanDelete;
    property Control: TcxSideBar read GetControl;
    property ListBox: TListBox read GetListBox;
  public
    constructor Create(ADesigner: TcxSideBarDesigner);
    procedure AddNewItems; virtual; abstract;
    procedure DeleteSelection; virtual;
    procedure Refresh;
    procedure RefreshEnableds; virtual;
    function UniqueName: string;
    property Count: Integer read GetCount;
    property Designer: TcxSideBarDesigner read FDesigner;
    property Items[Index: Integer]: TPersistent read GetItem; default;
    property SelectedCount: Integer read GetSelectedCount;
    property SelectedItem: TPersistent read GetSelectedItem write SetSelectedItem;
    property SelectedItems: TDesignerSelection read GetSelectedItems write SetSelectedItems;
  end;

  TcxSideBarDesignerComponents = class(TcxSideBarDesignerObjects)
  protected
    function CreateComponent: TComponent; virtual; abstract;
  public
    procedure AddNewItems; override;
  end;

  TcxSideBarDesignerGroups = class(TcxSideBarDesignerComponents)
  protected
    function ChangeItemIndex(AItem: TPersistent; NewIndex: Integer): Integer; override;
    function CreateComponent: TComponent; override;
    function GetBaseName: string; override;
    function GetButtonAdd: TButton; override;
    function GetButtonDelete: TButton; override;
    function GetCount: Integer; override;
    function GetItem(Index: Integer): TPersistent; override;
    function GetListBox: TListBox; override;
    procedure SelectionChanged; override;
  public
    procedure DeleteSelection; override;
    procedure RefreshEnableds; override;
  end;

  TcxSideBarDesignerItems = class(TcxSideBarDesignerComponents)
  protected
    function ChangeItemIndex(AItem: TPersistent; NewIndex: Integer): Integer; override;
    function CreateComponent: TComponent; override;
    function GetBaseName: string; override;
    function GetButtonAdd: TButton; override;
    function GetButtonDelete: TButton; override;
    function GetCount: Integer; override;
    function GetItem(Index: Integer): TPersistent; override;
    function GetListBox: TListBox; override;
    procedure SelectionChanged; override;
  public
    procedure DeleteSelection; override;
    procedure RefreshEnableds; override;
  end;

  TcxSideBarDesignerLinks = class(TcxSideBarDesignerObjects)
  private
    function GetGroup: TcxSideBarGroup;
  protected
    function ChangeItemIndex(AItem: TPersistent; NewIndex: Integer): Integer; override;
    function GetButtonAdd: TButton; override;
    function GetButtonDelete: TButton; override;
    function GetCount: Integer; override;
    function GetDisplayText(AObject: TPersistent): string; override;
    function GetItem(Index: Integer): TPersistent; override;
    function GetListBox: TListBox; override;
  public
    procedure AddNewItems; override;
    procedure RefreshEnableds; override;
    property Group: TcxSideBarGroup read GetGroup;
  end;

  TMyDesignWindow = {$IFDEF VCL}TDesignWindow{$ELSE}TClxDesignWindow{$ENDIF};

  TcxSideBarDesigner = class(TMyDesignWindow)
    gbxGroups: TGroupBox;
    lbxGroups: TListBox;
    gbxLinks: TGroupBox;
    lbxLinks: TListBox;
    gbxItems: TGroupBox;
    lbxItems: TListBox;
    btnLinkAdd: TButton;
    btnGroupAdd: TButton;
    btnGroupDelete: TButton;
    btnItemAdd: TButton;
    btnItemDelete: TButton;
    btnLinkDelete: TButton;
    procedure btnGroupAddClick(Sender: TObject);
    procedure btnGroupDeleteClick(Sender: TObject);
    procedure btnItemAddClick(Sender: TObject);
    procedure btnItemDeleteClick(Sender: TObject);
    procedure btnLinkAddClick(Sender: TObject);
    procedure btnLinkDeleteClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormShow(Sender: TObject);
  private
    FControl: TcxSideBar;
    FGroups: TcxSideBarDesignerGroups;
    FItems: TcxSideBarDesignerItems;
    FLinks: TcxSideBarDesignerLinks;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
    function UniqueName(Component: TComponent): string; override;

    procedure PrepareData;
    procedure RefreshCaption;
    procedure SelectComponent(AComponent: TComponent);
    procedure SelectControl;

    procedure ControlNameChanged;
    procedure GroupNameChanged;
    procedure ItemNameChanged;
    procedure LinkItemChanged;

    property Groups: TcxSideBarDesignerGroups read FGroups;
    property Items: TcxSideBarDesignerItems read FItems;
    property Links: TcxSideBarDesignerLinks read FLinks;
  public
    constructor Create(AControl: TcxSideBar); reintroduce;
    destructor Destroy; override;
  {$IFDEF DELPHI6}
    procedure SelectionChanged(const ADesigner: IDesigner;
      const ASelection: IDesignerSelections); override;
  {$ELSE}
    procedure SelectionChanged(ASelection: TDesignerSelectionList); override;
  {$ENDIF}

    procedure ShowInFront;
    property Control: TcxSideBar read FControl;
  end;

var
  cxSideBarDesigners: TcxSideBarDesigners;

implementation

{$R *.dfm}

uses
  SysUtils{$IFNDEF VCL}, Qt, QDialogs, cxControls{$ENDIF};

{$IFNDEF DELPHI6}
function CreateSelectionList: TDesignerSelectionList;
begin
  Result := TDesignerSelectionList.Create;
end;
{$ENDIF}

function DesignerSelectionListContains(List: TDesignerSelection;
  Item: TPersistent): Boolean;
var
  I: Integer;
begin
  for I := 0 to List.Count - 1 do
  begin
    Result := List[I] = Item;
    if Result then Exit;
  end;
  Result := False;
end;

function GetIfOnlyOne(AList: TDesignerSelection): TPersistent;
begin
  if AList.Count = 1 then
    Result := AList[0]
  else
    Result := nil;
end;

function GetListBoxSelectedObjects(AListBox: TListBox): TDesignerSelection;
var
  I: Integer;
begin
  Result := CreateSelectionList;
  with AListBox do
    for I := 0 to Items.Count - 1 do
      if Selected[I] then
        Result.Add(TPersistent(Items.Objects[I]));
end;

procedure SetListBoxSelectedObjects(AListBox: TListBox;
  ASelectedObjects: TDesignerSelection);
var
  I: Integer;
begin
  with AListBox do
  begin
    Items.BeginUpdate;
    try
      for I := 0 to Items.Count - 1 do
        Selected[I] :=
          DesignerSelectionListContains(ASelectedObjects, TPersistent(Items.Objects[I]));
    finally
      Items.EndUpdate;
    end;
  end;
end;

{ TcxSideBarDesigners }

constructor TcxSideBarDesigners.Create;
begin
  inherited;
  FItems := TList.Create;
end;

destructor TcxSideBarDesigners.Destroy;
begin
  FItems.Free;
  inherited;
end;

function TcxSideBarDesigners.GetCount: Integer;
begin
  Result := FItems.Count;
end;

function TcxSideBarDesigners.GetDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
begin
  Result := FindDesigner(AControl);
  if Result = nil then
    Result := CreateDesigner(AControl);
end;

function TcxSideBarDesigners.GetIsCustomizing(AControl: TcxSideBar): Boolean;
begin
  Result := FindDesigner(AControl) <> nil;
end;

function TcxSideBarDesigners.GetItem(Index: Integer): TcxSideBarDesigner;
begin
  Result := TcxSideBarDesigner(FItems[Index]);
end;

function TcxSideBarDesigners.CreateDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
begin
  Result := TcxSideBarDesigner.Create(AControl);
end;

function TcxSideBarDesigners.FindDesigner(AControl: TcxSideBar): TcxSideBarDesigner;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
  begin
    Result := Items[I];
    if Result.Control = AControl then Exit;
  end;
  Result := nil;
end;

procedure TcxSideBarDesigners.DoDesignerCreated(ADesigner: TcxSideBarDesigner);
begin
  FItems.Add(ADesigner);
  if (LastPos.X = 0) and (LastPos.Y = 0) then
    ADesigner.Position := poScreenCenter
  else
    with ADesigner, LastPos do
    begin
      Left := X;
      Top := Y;
    end;
end;

procedure TcxSideBarDesigners.DoDesignerDeleted(ADesigner: TcxSideBarDesigner);
begin
  with ADesigner do
    LastPos := Point(Left, Top);
  FItems.Remove(ADesigner);
end;

procedure TcxSideBarDesigners.ControlNameChanged(AControl: TcxSideBar);
begin
  if IsCustomizing[AControl] then
    Designers[AControl].ControlNameChanged;
end;

procedure TcxSideBarDesigners.GroupNameChanged(AControl: TcxSideBar);
begin
  if IsCustomizing[AControl] then
    Designers[AControl].GroupNameChanged;
end;

procedure TcxSideBarDesigners.ItemNameChanged(AControl: TcxSideBar);
begin
  if IsCustomizing[AControl] then
    Designers[AControl].ItemNameChanged;
end;

procedure TcxSideBarDesigners.LinkItemChanged(AControl: TcxSideBar);
begin
  if IsCustomizing[AControl] then
    Designers[AControl].LinkItemChanged;
end;

procedure TcxSideBarDesigners.ShowDesigner(AControl: TcxSideBar; ADesigner: TDesigner);
begin
  with Designers[AControl] do
  begin
    Designer := ADesigner;
    ShowInFront;
  end;
end;

{ TcxSideBarDesignerObjects }

constructor TcxSideBarDesignerObjects.Create(ADesigner: TcxSideBarDesigner);
begin
  inherited Create;
  FCanAdd := True;
  FDesigner := ADesigner;
  with ListBox do
  begin
    OnClick := ListBoxClick;
    OnDragDrop := ListBoxDragDrop;
    OnDragOver := ListBoxDragOver;
    OnKeyDown := ListBoxKeyDown;
    OnStartDrag := ListBoxStartDrag;
  end;
end;

function TcxSideBarDesignerObjects.GetControl: TcxSideBar;
begin
  Result := Designer.Control;
end;

function TcxSideBarDesignerObjects.GetSelectedCount: Integer;
var
  ASelectedItems: TDesignerSelection;
begin
  ASelectedItems := SelectedItems;
  try
    Result := ASelectedItems.Count;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
  end;
end;

function TcxSideBarDesignerObjects.GetSelectedItem: TPersistent;
var
  ASelectedItems: TDesignerSelection;
begin
  ASelectedItems := SelectedItems;
  try
    Result := GetIfOnlyOne(ASelectedItems);
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
  end;
end;

function TcxSideBarDesignerObjects.GetSelectedItems: TDesignerSelection;
begin
  Result := GetListBoxSelectedObjects(ListBox);
end;

procedure TcxSideBarDesignerObjects.SetSelectedItem(Value: TPersistent);
var
  ASelectedItems: TDesignerSelection;
begin
  ASelectedItems := CreateSelectionList;
  try
    ASelectedItems.Add(Value);
    SelectedItems := ASelectedItems;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
  end;
end;

procedure TcxSideBarDesignerObjects.SetSelectedItems(Value: TDesignerSelection);
var
  ASelectedItems: TDesignerSelection;
begin
  ASelectedItems := SelectedItems;
  try
    if not ASelectedItems.Equals(Value) then
    begin
      SetListBoxSelectedObjects(ListBox, Value);
      SelectionChanged;
    end;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
  end;
end;

procedure TcxSideBarDesignerObjects.ListBoxClick(Sender: TObject);
begin
  SelectionChanged;
  if SelectedCount = 0 then
    Designer.SelectControl
  else
    Designer.SetSelection(SelectedItems);
end;

procedure TcxSideBarDesignerObjects.ListBoxDragDrop(Sender, Source: TObject;
  X, Y: Integer);
var
  NewIndex, I: Integer;
  ASelectedItems: TDesignerSelection;
begin
  NewIndex := ListBox.ItemAtPos(Point(X, Y), False);
  ASelectedItems := SelectedItems;
  try
    for I := 0 to ASelectedItems.Count - 1 do
    begin
      if NewIndex > ListBox.Items.IndexOfObject(ASelectedItems[I]) then
        Dec(NewIndex);
      NewIndex := ChangeItemIndex(ASelectedItems[I], NewIndex) + 1;
    end;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
    Refresh;
    ListBox.ItemIndex := NewIndex - SelectedCount;
    Designer.Designer.Modified;
  end;
end;

procedure TcxSideBarDesignerObjects.ListBoxDragOver(Sender, Source: TObject;
  X, Y: Integer; State: TDragState; var Accept: Boolean);
begin
  Accept := (Source = Sender) and (SelectedCount <> 0);
  if Accept and (State = dsDragMove) then
    ListBox.ItemIndex := ListBox.ItemAtPos(Point(X, Y), True)
  else
    ListBox.ItemIndex := -1;
end;

procedure TcxSideBarDesignerObjects.ListBoxKeyDown(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_INSERT:
      if CanAdd then AddNewItems;
    VK_DELETE:
      if CanDelete then DeleteSelection;
  end;
end;

procedure TcxSideBarDesignerObjects.ListBoxStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  ListBox.SetFocus;
end;

function TcxSideBarDesignerObjects.GetBaseName: string;
begin
  Result := Control.Name;
end;

function TcxSideBarDesignerObjects.GetDisplayText(AObject: TPersistent): string;
begin
  Result := (AObject as TComponent).Name;
end;

procedure TcxSideBarDesignerObjects.SelectionChanged;
begin
  RefreshEnableds;
end;

procedure TcxSideBarDesignerObjects.DeleteSelection;
var
  ASelectedItems: TDesignerSelection;
  I: Integer;
begin
  Designer.SelectControl;
  ASelectedItems := SelectedItems;
  try
    for I := 0 to ASelectedItems.Count - 1 do
      ASelectedItems[I].Free;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
    Refresh;
    Designer.Designer.Modified;
  end;
end;

procedure TcxSideBarDesignerObjects.Refresh;
var
  FocusedIndex: Integer;
  ASelectedItems: TDesignerSelection;
  AObject: TPersistent;

  procedure RefreshItems;
  var
    I: Integer;
  begin
    with ListBox.Items do
    begin
      BeginUpdate;
      try
        Clear;
        for I := 0 to Self.Count - 1 do
        begin
          AObject := Self.Items[I];
          AddObject(GetDisplayText(AObject), AObject);
          if DesignerSelectionListContains(ASelectedItems, AObject) then
            ListBox.Selected[I] := True;
        end;
      finally
        EndUpdate;
      end;
    end;
  end;

begin
  FocusedIndex := ListBox.ItemIndex;
  try
    ASelectedItems := SelectedItems;
    RefreshItems;
  finally
    {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
    if FocusedIndex < ListBox.Items.Count then
      ListBox.ItemIndex := FocusedIndex;
    RefreshEnableds;
  end;
end;

procedure TcxSideBarDesignerObjects.RefreshEnableds;
begin
  CanDelete := SelectedCount <> 0;
  ButtonAdd.Enabled := CanAdd;
  ButtonDelete.Enabled := CanDelete;
end;

function TcxSideBarDesignerObjects.UniqueName: string;
var
  NameIndex, I: Integer;
  Found: Boolean;
begin
  NameIndex := 0;
  repeat
    Inc(NameIndex);
    Result := BaseName + IntToStr(NameIndex);
    Found := False;
    for I := 0 to Count - 1 do
    begin
      Found := SameText(TComponent(Items[I]).Name, Result);
      if Found then Break;
    end;
  until not Found;
end;

{ TcxSideBarDesignerComponents }

procedure TcxSideBarDesignerComponents.AddNewItems;
var
  AComponent: TComponent;
begin
  AComponent := CreateComponent;
  AComponent.Name := UniqueName;
  Refresh;
  SelectedItem := AComponent;
  Designer.SelectComponent(AComponent);
end;

{ TcxSideBarDesignerGroups }

function TcxSideBarDesignerGroups.ChangeItemIndex(AItem: TPersistent;
  NewIndex: Integer): Integer;
begin
  TcxSideBarGroup(AItem).Index := NewIndex;
  Result := TcxSideBarGroup(AItem).Index;
end;

function TcxSideBarDesignerGroups.CreateComponent: TComponent;
begin
  Result := Designer.Control.AddGroup;
  TcxSideBarGroup(Result).Caption := 'New Group';
end;

function TcxSideBarDesignerGroups.GetBaseName: string;
begin
  Result := inherited GetBaseName + 'Group';
end;

function TcxSideBarDesignerGroups.GetButtonAdd: TButton;
begin
  Result := Designer.btnGroupAdd;
end;

function TcxSideBarDesignerGroups.GetButtonDelete: TButton;
begin
  Result := Designer.btnGroupDelete;
end;

function TcxSideBarDesignerGroups.GetCount: Integer;
begin
  Result := Control.GroupCount;
end;

function TcxSideBarDesignerGroups.GetItem(Index: Integer): TPersistent;
begin
  Result := Control.Groups[Index];
end;

function TcxSideBarDesignerGroups.GetListBox: TListBox;
begin
  Result := Designer.lbxGroups;
end;

procedure TcxSideBarDesignerGroups.SelectionChanged;
begin
  inherited;
  Designer.Links.Refresh;
end;

procedure TcxSideBarDesignerGroups.DeleteSelection;
begin
  inherited;
  Designer.Links.Refresh;
end;

procedure TcxSideBarDesignerGroups.RefreshEnableds;
begin
  inherited;
  Designer.Links.RefreshEnableds;
end;

{ TcxSideBarDesignerItems }

function TcxSideBarDesignerItems.ChangeItemIndex(AItem: TPersistent;
  NewIndex: Integer): Integer;
begin
  TcxSideBarItem(AItem).Index := NewIndex;
  Result := TcxSideBarItem(AItem).Index;
end;

function TcxSideBarDesignerItems.CreateComponent: TComponent;
begin
  Result := Designer.Control.AddItem;
  TcxSideBarItem(Result).Caption := 'New Item';
end;

function TcxSideBarDesignerItems.GetBaseName: string;
begin
  Result := inherited GetBaseName + 'Item';
end;

function TcxSideBarDesignerItems.GetButtonAdd: TButton;
begin
  Result := Designer.btnItemAdd;
end;

function TcxSideBarDesignerItems.GetButtonDelete: TButton;
begin
  Result := Designer.btnItemDelete;
end;

function TcxSideBarDesignerItems.GetCount: Integer;
begin
  Result := Control.ItemCount;
end;

function TcxSideBarDesignerItems.GetItem(Index: Integer): TPersistent;
begin
  Result := Control.Items[Index];
end;

function TcxSideBarDesignerItems.GetListBox: TListBox;
begin
  Result := Designer.lbxItems;
end;

procedure TcxSideBarDesignerItems.SelectionChanged;
begin
  inherited;
  Designer.Links.RefreshEnableds;
end;

procedure TcxSideBarDesignerItems.DeleteSelection;
begin
  inherited;
  Designer.Links.Refresh;
end;

procedure TcxSideBarDesignerItems.RefreshEnableds;
begin
  inherited;
  Designer.Links.RefreshEnableds;
end;

{ TcxSideBarDesignerLinks }

function TcxSideBarDesignerLinks.ChangeItemIndex(AItem: TPersistent;
  NewIndex: Integer): Integer;
begin
  TcxSideBarLink(AItem).Index := NewIndex;
  Result := TcxSideBarLink(AItem).Index;
end;

function TcxSideBarDesignerLinks.GetGroup: TcxSideBarGroup;
begin
  Result := TcxSideBarGroup(Designer.Groups.SelectedItem);
end;

function TcxSideBarDesignerLinks.GetButtonAdd: TButton;
begin
  Result := Designer.btnLinkAdd;
end;

function TcxSideBarDesignerLinks.GetButtonDelete: TButton;
begin
  Result := Designer.btnLinkDelete;
end;

function TcxSideBarDesignerLinks.GetCount: Integer;
begin
  if Group = nil then
    Result := 0
  else
    Result := Group.Links.Count;
end;

function TcxSideBarDesignerLinks.GetDisplayText(AObject: TPersistent): string;

  function GetName: string;
  begin
    with TcxSideBarLink(AObject) do
      if Item = nil then
        Result := 'none'
      else
        Result := Item.Name;
  end;

begin
  Result := '( ' + GetName + ' )';
end;

function TcxSideBarDesignerLinks.GetItem(Index: Integer): TPersistent;
begin
  Result := Group.Links[Index];
end;

function TcxSideBarDesignerLinks.GetListBox: TListBox;
begin
  Result := Designer.lbxLinks;
end;

procedure TcxSideBarDesignerLinks.AddNewItems;
var
  ASelectedLinks, ASelectedItems: TDesignerSelection;
  I: Integer;
begin
  ASelectedLinks := CreateSelectionList;
  try
    ASelectedItems := Designer.Items.SelectedItems;
    try
      for I := 0 to ASelectedItems.Count - 1 do
        ASelectedLinks.Add(Group.AddLink(TcxSideBarItem(ASelectedItems[I])));
    finally
      {$IFNDEF DELPHI6}ASelectedItems.Free;{$ENDIF}
    end;
  finally
    Refresh;
    SelectedItems := ASelectedLinks;
    Designer.SetSelection(ASelectedLinks);
    Designer.Designer.Modified;
  end;
end;

procedure TcxSideBarDesignerLinks.RefreshEnableds;
begin
  CanAdd := (Group <> nil) and (Designer.Items.SelectedCount <> 0);
  inherited;
end;

{ TcxSideBarDesigner }

constructor TcxSideBarDesigner.Create(AControl: TcxSideBar);
begin
  inherited Create(nil);
  FControl := AControl;
  FControl.FreeNotification(Self);
  FGroups := TcxSideBarDesignerGroups.Create(Self);
  FItems := TcxSideBarDesignerItems.Create(Self);
  FLinks := TcxSideBarDesignerLinks.Create(Self);
  cxSideBarDesigners.DoDesignerCreated(Self);
end;

destructor TcxSideBarDesigner.Destroy;
begin
  if not (csDestroying in Control.ComponentState) then
    SelectControl;
  cxSideBarDesigners.DoDesignerDeleted(Self);
  FLinks.Free;
  FItems.Free;
  FGroups.Free;
  inherited;
end;

procedure TcxSideBarDesigner.Notification(AComponent: TComponent; Operation: TOperation);
begin
  inherited;
  if (Operation = opRemove) and (AComponent = Control) then
    Free;
end;

function TcxSideBarDesigner.UniqueName(Component: TComponent): string;
begin
  Result := '';
end;

procedure TcxSideBarDesigner.PrepareData;
begin
  RefreshCaption;
  Groups.Refresh;
  Items.Refresh;
end;

procedure TcxSideBarDesigner.RefreshCaption;
begin
  Caption := 'Designer - ' + Control.Name;
end;

procedure TcxSideBarDesigner.SelectComponent(AComponent: TComponent);
var
  ASelection: TDesignerSelection;
begin
  ASelection := CreateSelectionList;
  ASelection.Add(AComponent);
  SetSelection(ASelection);
end;

procedure TcxSideBarDesigner.SelectControl;
begin
  SelectComponent(Control);
end;

procedure TcxSideBarDesigner.ControlNameChanged;
begin
  RefreshCaption;
end;

procedure TcxSideBarDesigner.GroupNameChanged;
begin
  Groups.Refresh;
end;

procedure TcxSideBarDesigner.ItemNameChanged;
begin
  Items.Refresh;
  Links.Refresh;
end;

procedure TcxSideBarDesigner.LinkItemChanged;
begin
  Links.Refresh;
end;

{$IFDEF DELPHI6}
procedure TcxSideBarDesigner.SelectionChanged(const ADesigner: IDesigner;
  const ASelection: IDesignerSelections);
{$ELSE}
procedure TcxSideBarDesigner.SelectionChanged(ASelection: TDesignerSelectionList);
{$ENDIF}
begin
  inherited;
  if (ASelection = nil) or (ASelection.Count = 0) or (ASelection[0] is TcxSideBar) then
    //
  else
    if ASelection[0] is TcxSideBarGroup then
      Groups.SelectedItems := ASelection
    else
      if ASelection[0] is TcxSideBarItem then
        Items.SelectedItems := ASelection
      else
        if ASelection[0] is TcxSideBarLinks then
          Links.SelectedItems := ASelection;
end;

procedure TcxSideBarDesigner.ShowInFront;
begin
  Show;
  BringToFront;
  SelectControl;
end;

procedure TcxSideBarDesigner.btnGroupAddClick(Sender: TObject);
begin
  Groups.AddNewItems;
end;

procedure TcxSideBarDesigner.btnGroupDeleteClick(Sender: TObject);
begin
  Groups.DeleteSelection;
end;

procedure TcxSideBarDesigner.btnItemAddClick(Sender: TObject);
begin
  Items.AddNewItems;
end;

procedure TcxSideBarDesigner.btnItemDeleteClick(Sender: TObject);
begin
  Items.DeleteSelection;
end;

procedure TcxSideBarDesigner.btnLinkAddClick(Sender: TObject);
begin
  Links.AddNewItems;
end;

procedure TcxSideBarDesigner.btnLinkDeleteClick(Sender: TObject);
begin
  Links.DeleteSelection;
end;

procedure TcxSideBarDesigner.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Action := caFree;
end;

procedure TcxSideBarDesigner.FormShow(Sender: TObject);
begin
  PrepareData;
end;

initialization
  cxSideBarDesigners := TcxSideBarDesigners.Create;

finalization
  cxSideBarDesigners.Free;

end.
