{**********************************************************************}
{                                                                      }
{       Developer Express Visual Component Library                     }
{       ExpressSideBar - CLX/VCL Edition                               }
{                                                                      }
{       Copyright (c) 1998-2001 Developer Express Inc.                 }
{       ALL RIGHTS RESERVED                                            }
{                                                                      }
{   The entire contents of this file is protected by U.S. and          }
{   International Copyright Laws. Unauthorized reproduction,           }
{   reverse-engineering, and distribution of all or any portion of     }
{   the code contained in this file is strictly prohibited and may     }
{   result in severe civil and criminal penalties and will be          }
{   prosecuted to the maximum extent possible under the law.           }
{                                                                      }
{   RESTRICTIONS                                                       }
{                                                                      }
{   THIS SOURCE CODE AND ALL RESULTING INTERMEDIATE FILES              }
{   (DCU, OBJ, DLL, DPU, SO, ETC.) ARE CONFIDENTIAL AND PROPRIETARY    }
{   TRADE SECRETS OF DEVELOPER EXPRESS INC. THE REGISTERED DEVELOPER   }
{   IS LICENSED TO DISTRIBUTE THE EXPRESSSIDEBAR                       }
{   AS PART OF AN EXECUTABLE PROGRAM ONLY.                             }
{                                                                      }
{   THE SOURCE CODE CONTAINED WITHIN THIS FILE AND ALL RELATED         }
{   FILES OR ANY PORTION OF ITS CONTENTS SHALL AT NO TIME BE           }
{   COPIED, TRANSFERRED, SOLD, DISTRIBUTED, OR OTHERWISE MADE          }
{   AVAILABLE TO OTHER INDIVIDUALS WITHOUT EXPRESS WRITTEN CONSENT     }
{   AND PERMISSION FROM DEVELOPER EXPRESS INC.                         }
{                                                                      }
{   CONSULT THE END USER LICENSE AGREEMENT FOR INFORMATION ON          }
{   ADDITIONAL RESTRICTIONS.                                           }
{                                                                      }
{**********************************************************************}

unit cxSideBarReg;

{$I cxSideBarVer.inc}

interface

procedure Register;

implementation

uses
  SysUtils, Classes, TypInfo,
  {$IFDEF DELPHI6}DesignIntf, DesignEditors{$ELSE}DsgnIntf, Windows, Forms{$ENDIF},
  cxSideBar, cxSideBarDesigner;

type
  TFormDesigner = {$IFDEF DELPHI6}IDesigner{$ELSE}IFormDesigner{$ENDIF};

{ TcxSideBarRealDesignerLink }

type
  TcxSideBarRealDesignerLink = class(TcxSideBarDesignerLink)
  public
    procedure ControlNameChanged(AControl: TcxSideBar); override;
    procedure GroupNameChanged(AControl: TcxSideBar); override;
    procedure ItemNameChanged(AControl: TcxSideBar); override;
    procedure LinkItemChanged(AControl: TcxSideBar); override;
    procedure SetSelection(AControl: TcxSideBar; AObject: TPersistent); override;
    procedure ShowDefaultEventHandler(AItem: TcxSideBarItem); override;
  end;

procedure TcxSideBarRealDesignerLink.ControlNameChanged(AControl: TcxSideBar);
begin
  cxSideBarDesigners.ControlNameChanged(AControl);
end;

procedure TcxSideBarRealDesignerLink.GroupNameChanged(AControl: TcxSideBar);
begin
  cxSideBarDesigners.GroupNameChanged(AControl);
end;

procedure TcxSideBarRealDesignerLink.ItemNameChanged(AControl: TcxSideBar);
begin
  cxSideBarDesigners.ItemNameChanged(AControl);
end;

procedure TcxSideBarRealDesignerLink.LinkItemChanged(AControl: TcxSideBar);
begin
  cxSideBarDesigners.LinkItemChanged(AControl);
end;

procedure TcxSideBarRealDesignerLink.SetSelection(AControl: TcxSideBar; AObject: TPersistent);
begin
{$IFDEF VCL}
  (GetDesigner(AControl) as TFormDesigner).SelectComponent(AObject);
{$ENDIF}
end;

procedure TcxSideBarRealDesignerLink.ShowDefaultEventHandler(AItem: TcxSideBarItem);
{$IFDEF VCL}
var
  APropInfo: PPropInfo;
  AFormDesigner: TFormDesigner;
  AMethod: TMethod;
  AMethodName: string;
{$ENDIF}  
begin
{$IFDEF VCL}
  APropInfo := GetPropInfo(AItem.ClassInfo, 'OnClick');
  if APropInfo = nil then
    Exit
  else
    AMethodName := 'Click';
  AFormDesigner := GetDesigner(AItem.Control) as TFormDesigner;
  AMethod := GetMethodProp(AItem, APropInfo);
  if AMethod.Code <> nil then
    AMethodName := AFormDesigner.GetMethodName(AMethod)
  else
  begin
    AMethodName := AItem.Name + AMethodName;
    AMethod :=
      AFormDesigner.CreateMethod(AMethodName, GetTypeData(APropInfo^.PropType^));
    SetMethodProp(AItem, APropInfo, AMethod);
    AFormDesigner.Modified;
  end;
  AFormDesigner.ShowMethod(AMethodName);
{$ENDIF}  
end;

{ TSideBarComponentEditor }

type
  TSideBarComponentEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TSideBarComponentEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
    0: cxSideBarDesigners.ShowDesigner(TcxSideBar(Component), Designer);
  end;
end;

function TSideBarComponentEditor.GetVerb(Index: Integer): string;
begin
  case Index of
    0: Result := 'Designer...';
    1: Result := '-';
    2: Result := 'CrossPlatform ExpressSideBar 1.0';
    3: Result := 'Developer Express Inc.';
  else
    Result := '';
  end;
end;

function TSideBarComponentEditor.GetVerbCount: Integer;
begin
  Result := 1 + 3;
end;

{ TLinksProperty }

type
  TLinksProperty = class(TPropertyEditor)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
    function GetValue: string; override;
  end;

procedure TLinksProperty.Edit;
var
  Control: TcxSideBar;
begin
  Control := TcxSideBarGroup(GetComponent(0)).Control;
  if not cxSideBarDesigners.IsCustomizing[Control] then
    cxSideBarDesigners.ShowDesigner(Control, Designer);
end;

function TLinksProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paReadOnly];
end;

function TLinksProperty.GetValue: string;
begin
  Result := Format('(%s)', [GetPropType^.Name]);
end;

procedure Register;
begin
  RegisterComponentEditor(TcxSideBar, TSideBarComponentEditor);
  RegisterPropertyEditor(TypeInfo(TcxSideBarLinks), TcxSideBarGroup, '', TLinksProperty);

  RegisterNoIcon([TcxSideBarItem, TcxSideBarGroup]);
  RegisterComponents('Dev Express', [TcxSideBar]);
end;

initialization
  cxSideBarDesignerLink.Free;
  cxSideBarDesignerLink := TcxSideBarRealDesignerLink.Create;

finalization
  cxSideBarDesignerLink.Free;
  cxSideBarDesignerLink := nil;

end.
